# Based on iwidgets2.2.0/buttonbox.itk code.

import types
import Tkinter
import Pmw

class ButtonBox(Pmw.MegaWidget):
    def __init__(self, parent = None, **kw):

	# Define the megawidget options.
	INITOPT = Pmw.INITOPT
	optiondefs = (
	    ('labelmargin',       0,              INITOPT),
	    ('labelpos',          None,           INITOPT),
	    ('orient',            'horizontal',   INITOPT),
	    ('padx',              3,              INITOPT),
	    ('pady',              3,              INITOPT),
	)
	self.defineoptions(kw, optiondefs, dynamicGroups = ('Button',))

	# Initialise the base class (after defining the options).
	Pmw.MegaWidget.__init__(self, parent)

	# Create the components.
	interior = self.interior()
	if self['labelpos'] is None:
	    self._buttonBoxFrame = self._hull
	    columnOrRow = 0
	else:
	    self._buttonBoxFrame = self.createcomponent('frame',
		    (), None,
		    Tkinter.Frame, (interior,))
	    self._buttonBoxFrame.grid(column=2, row=2, sticky='nsew')
	    columnOrRow = 2

	    self.createlabel(interior)

	orient = self['orient']
	if orient == 'horizontal':
	    interior.grid_columnconfigure(columnOrRow, weight = 1)
	elif orient == 'vertical':
	    interior.grid_rowconfigure(columnOrRow, weight = 1)
	else:
	    raise ValueError, 'bad orient option ' + repr(orient) + \
		': must be either \'horizontal\' or \'vertical\''

	# Initialise instance variables.

	# List of tuples describing the buttons:
	#   - name
	#   - button widget
	self._buttonList = []

	# The index of the default button.
	self._defaultButton = None

	self._timerId = None

	# Check keywords and initialise options.
	self.initialiseoptions()

    def destroy(self):
	if self._timerId:
	    self.after_cancel(self._timerId)
	    self._timerId = None
	Pmw.MegaWidget.destroy(self)

    def numbuttons(self):
        return len(self._buttonList)

    def index(self, index, forInsert = 0):
	listLength = len(self._buttonList)
	if type(index) == types.IntType:
	    if forInsert and index <= listLength:
		return index
	    elif not forInsert and index < listLength:
		return index
	    else:
		raise ValueError, 'index "%s" is out of range' % index
	elif index is Pmw.END:
	    if forInsert:
		return listLength
	    elif listLength > 0:
		return listLength - 1
	    else:
		raise ValueError, 'ButtonBox has no buttons'
	elif index is Pmw.DEFAULT:
	    if self._defaultButton is not None:
		return self._defaultButton
	    raise ValueError, 'ButtonBox has no default'
	else:
            names = map(lambda t: t[0], self._buttonList)
            if index in names:
                return names.index(index)
	    validValues = 'a name, a number, Pmw.END or Pmw.DEFAULT'
	    raise ValueError, \
		'bad index "%s": must be %s' % (index, validValues)

    def insert(self, componentName, beforeComponent = 0, **kw):
	if componentName in self.components():
	    raise ValueError, 'button "%s" already exists' % componentName
	if not kw.has_key('text'):
	    kw['text'] = componentName
        kw['default'] = 'normal'
	button = apply(self.createcomponent, (componentName,
		(), 'Button',
		Tkinter.Button, (self._buttonBoxFrame,)), kw)

	index = self.index(beforeComponent, 1)
	horizontal = self['orient'] == 'horizontal'
	numButtons = len(self._buttonList)

	# Shift buttons up one position.
	for i in range(numButtons - 1, index - 1, -1):
	    widget = self._buttonList[i][1]
	    pos = i * 2 + 3
	    if horizontal:
		widget.grid(column = pos, row = 0)
	    else:
		widget.grid(column = 0, row = pos)

	# Display the new button.
	if horizontal:
	    button.grid(column = index * 2 + 1, row = 0, sticky = 'ew',
		    padx = self['padx'], pady = self['pady'])
	    self._buttonBoxFrame.grid_columnconfigure(
		    numButtons * 2 + 2, weight = 1)
	else:
	    button.grid(column = 0, row = index * 2 + 1, sticky = 'ew',
		    padx = self['padx'], pady = self['pady'])
	    self._buttonBoxFrame.grid_rowconfigure(
		    numButtons * 2 + 2, weight = 1)
	self._buttonList.insert(index, (componentName, button))

	return button

    def add(self, componentName, **kw):
        return apply(self.insert, (componentName, len(self._buttonList)), kw)

    def delete(self, index):
        index = self.index(index)
	(name, widget) = self._buttonList[index]
	widget.grid_forget()
	self.destroycomponent(name)

	numButtons = len(self._buttonList)

	# Shift buttons down one position.
	horizontal = self['orient'] == 'horizontal'
	for i in range(index + 1, numButtons):
	    widget = self._buttonList[i][1]
	    pos = i * 2 - 1
	    if horizontal:
		widget.grid(column = pos, row = 0)
	    else:
		widget.grid(column = 0, row = pos)

	if horizontal:
	    self._buttonBoxFrame.grid_columnconfigure(numButtons * 2 - 1,
		    minsize = 0)
	    self._buttonBoxFrame.grid_columnconfigure(numButtons * 2, weight = 0)
	else:
	    self._buttonBoxFrame.grid_rowconfigure(numButtons * 2, weight = 0)
	del self._buttonList[index]

    def setdefault(self, index):
	# Turn off the default ring around the current default button.
	if self._defaultButton is not None:
	    button = self._buttonList[self._defaultButton][1]
	    button.configure(default = 'normal')
	    self._defaultButton = None

	# Turn on the default ring around the new default button.
	if index is not None:
	    index = self.index(index)
	    self._defaultButton = index
	    button = self._buttonList[index][1]
	    button.configure(default = 'active')

    def invoke(self, index = Pmw.DEFAULT, noFlash = 0):
	# Invoke the callback associated with the *index* button.  If
	# *noFlash* is not set, flash the button to indicate to the
	# user that something happened.

	button = self._buttonList[self.index(index)][1]
	if not noFlash:
	    state = button.cget('state')
	    relief = button.cget('relief')
	    button.configure(state = 'active', relief = 'sunken')
	    self.update_idletasks()
	    self.after(100)
	    button.configure(state = state, relief = relief)
	return button.invoke()

    def button(self, buttonIndex):
	return self._buttonList[self.index(buttonIndex)][1]

    def alignbuttons(self, when = 'later'):
	if when == 'later':
	    if not self._timerId:
		self._timerId = self.after_idle(self.alignbuttons, 'now')
	    return
	self.update_idletasks()
	self._timerId = None

        # Determine the width of the maximum length button.
        max = 0
        horizontal = (self['orient'] == 'horizontal')
        for index in range(len(self._buttonList)):
            gridIndex = index * 2 + 1
            if horizontal:
                width = self._buttonBoxFrame.grid_bbox(gridIndex, 0)[2]
            else:
                width = self._buttonBoxFrame.grid_bbox(0, gridIndex)[2]
            if width > max:
                max = width

        # Set the width of all the buttons to be the same.
	if horizontal:
	    for index in range(len(self._buttonList)):
		self._buttonBoxFrame.grid_columnconfigure(index * 2 + 1,
			minsize = max)
	else:
	    self._buttonBoxFrame.grid_columnconfigure(0, minsize = max)



title = 'Pmw.EntryField demonstration'

# Import Pmw from this directory tree.
import sys
sys.path[:0] = ['../../..']

import time
import Tkinter
import Pmw

class Demo:
    def __init__(self, parent):
	# Create and pack the EntryFields.
	self._any = Pmw.EntryField(parent,
		labelpos = 'w',
		label_text = 'Any:',
		validate = None,
		command = self.execute)
	self._real = Pmw.EntryField(parent,
		labelpos = 'w',
		value = '55.5',
		label_text = 'Real (10.0 to 99.0):',
		validate = {'validator' : 'real',
			'min' : 10, 'max' : 99, 'minstrict' : 0},
		modifiedcommand = self.changed)
	self._odd = Pmw.EntryField(parent,
		labelpos = 'w',
		label_text = 'Odd length:',
		validate = self.custom_validate,
		value = 'ABC')
	self._date = Pmw.EntryField(parent,
		labelpos = 'w',
		label_text = 'Date (in 2000):',
		value = '2000/2/29',
		validate = {'validator' : 'date',
			'min' : '2000/1/1', 'max' : '2000/12/31',
			'minstrict' : 0, 'maxstrict' : 0,
			'format' : 'ymd'},
		)
	now = time.localtime(time.time())
	self._date2 = Pmw.EntryField(parent,
		labelpos = 'w',
		label_text = 'Date (d.m.y):',
		value = '%d.%d.%d' % (now[2], now[1], now[0]),
		validate = {'validator' : 'date',
			'format' : 'dmy', 'separator' : '.'},
		)
	self._time = Pmw.EntryField(parent,
		labelpos = 'w',
		label_text = 'Time (24hr clock):',
		value = '8:00:00',
		validate = {'validator' : 'time',
			'min' : '00:00:00', 'max' : '23:59:59',
			'minstrict' : 0, 'maxstrict' : 0},
		)
	self._comma = Pmw.EntryField(parent,
		labelpos = 'w',
		label_text = 'Real (with comma):',
		value = '123,456',
		validate = {'validator' : 'real', 'separator' : ','},
		)

	entries = (self._any, self._real, self._odd, self._date, self._date2,
		self._time, self._comma)

	for entry in entries:
	    entry.pack(fill='x', expand=1, padx=10, pady=5)
	Pmw.alignlabels(entries)

	self._any.component('entry').focus_set()

    def changed(self):
	print 'Text changed, value is', self._real.getvalue()

    def execute(self):
	print 'Return pressed, value is', self._any.getvalue()

    # This implements a custom validation routine.  It simply checks
    # if the string is of odd length.
    def custom_validate(self, text):
	print 'text:', text
	if len(text) % 2 == 0:
	  return -1
	else:
	  return 1

######################################################################

# Create demo in root window for testing.
if __name__ == '__main__':
    root = Tkinter.Tk()
    Pmw.initialise(root, fontScheme = 'pmw1')
    root.title(title)

    exitButton = Tkinter.Button(root, text = 'Exit', command = root.destroy)
    exitButton.pack(side = 'bottom')
    widget = Demo(root)
    root.mainloop()



title = 'Pmw.MessageBar demonstration'

# Import Pmw from this directory tree.
import sys
sys.path[:0] = ['../../..']

import Tkinter
import Pmw

class Demo:
    def __init__(self, parent):
	# Create and pack the MessageBar.
	self._messagebar = Pmw.MessageBar(parent,
		entry_width = 40,
		entry_relief='groove',
		labelpos = 'w',
		label_text = 'Status:')
	self._messagebar.pack(side = 'bottom', fill = 'x',
		expand = 1, padx = 10, pady = 10)

	# Create and pack the ScrolledListBox to change the MessageBar.
	self.box = Pmw.ScrolledListBox(parent,
		listbox_selectmode='single',
		items=('state', 'help', 'userevent', 'systemevent',
			'usererror', 'systemerror', 'busy',),
		label_text='Message type',
		labelpos='n',
		selectioncommand=self.selectionCommand)
	self.box.pack(fill = 'both', expand = 'yes', padx = 10, pady = 10)

	self._index = 0
	self._stateCounter = 0

    def selectionCommand(self):
	sels = self.box.getcurselection()
	if len(sels) > 0:
	    self._index = self._index + 1
	    messagetype = sels[0]
	    if messagetype == 'state':
		self._stateCounter = (self._stateCounter + 1) % 3
		text = stateMessages[self._stateCounter]
		if text != '':
		    text = text + ' (' + messagetype + ')'
		self._messagebar.message('state', text)
	    else:
		text = messages[messagetype]
		text = text + ' (' + messagetype + ')'
		self._messagebar.message(messagetype, text)
		if messagetype == 'busy':
		    Pmw.showbusycursor()
		    self.box.after(2000)
		    Pmw.hidebusycursor()
		    self._messagebar.resetmessages('busy')
		    text = 'All files successfully removed'
		    text = text + ' (userevent)'
		    self._messagebar.message('userevent', text)


messages = {
    'help': 'Save current file',
    'userevent': 'Saving file "foo"',
    'busy': 'Busy deleting all files from file system ...',
    'systemevent': 'File "foo" saved',
    'usererror': 'Invalid file name "foo/bar"',
    'systemerror': 'Failed to save file: file system full',
}

stateMessages = {
    0: '',
    1: 'Database is down',
    2: 'Waiting for reply from database',
}

######################################################################

# Create demo in root window for testing.
if __name__ == '__main__':
    root = Tkinter.Tk()
    Pmw.initialise(root, fontScheme = 'pmw1')
    root.title(title)

    exitButton = Tkinter.Button(root, text = 'Exit', command = root.destroy)
    exitButton.pack(side = 'bottom')
    widget = Demo(root)
    root.mainloop()



# Not Based on iwidgets version.

import Pmw

class SelectionDialog(Pmw.Dialog):
    # Dialog window with selection list.
    
    # Dialog window displaying a list and requesting the user to
    # select one.

    def __init__(self, parent = None, **kw):
	# Define the megawidget options.
	INITOPT = Pmw.INITOPT
	optiondefs = (
	    ('borderx',     10,    INITOPT),
	    ('bordery',     10,    INITOPT),
	)
	self.defineoptions(kw, optiondefs)

	# Initialise the base class (after defining the options).
	Pmw.Dialog.__init__(self, parent)

	# Create the components.
	interior = self.interior()
	aliases = (
	    ('listbox', 'scrolledlist_listbox'),
	    ('label', 'scrolledlist_label'),
	)
	self._list = self.createcomponent('scrolledlist',
		aliases, None,
		Pmw.ScrolledListBox, (interior,),
		dblclickcommand = self.invoke)
	self._list.pack(side='top', expand='true', fill='both',
		padx = self['borderx'], pady = self['bordery'])

        if not kw.has_key('activatecommand'):
            # Whenever this dialog is activated, set the focus to the
            # ScrolledListBox's listbox widget.
            listbox = self.component('listbox')
            self.configure(activatecommand = listbox.focus_set)

	# Check keywords and initialise options.
	self.initialiseoptions()

    # Need to explicitly forward this to override the stupid
    # (grid_)size method inherited from Tkinter.Toplevel.Grid.
    def size(self):
	return self.component('listbox').size()

    # Need to explicitly forward this to override the stupid
    # (grid_)bbox method inherited from Tkinter.Toplevel.Grid.
    def bbox(self, index):
	return self.component('listbox').size(index)

Pmw.forwardmethods(SelectionDialog, Pmw.ScrolledListBox, '_list')



# Based on iwidgets2.2.0/messagedialog.itk code.

import Tkinter
import Pmw

class MessageDialog(Pmw.Dialog):
    def __init__(self, parent = None, **kw):

	# Define the megawidget options.
	INITOPT = Pmw.INITOPT
	optiondefs = (
	    ('borderx',       20,    INITOPT),
	    ('bordery',       20,    INITOPT),
	    ('iconmargin',    20,    INITOPT),
	    ('iconpos',       None,  INITOPT),
	)
	self.defineoptions(kw, optiondefs)

	# Initialise the base class (after defining the options).
	Pmw.Dialog.__init__(self, parent)

	# Create the components.
	interior = self.interior()

	self._message = self.createcomponent('message',
		(), None,
		Tkinter.Label, (interior,))

        iconpos = self['iconpos']
	iconmargin = self['iconmargin']
	borderx = self['borderx']
	bordery = self['bordery']
	border_right = 2
	border_bottom = 2
	if iconpos is None:
	    self._message.grid(column = 1, row = 1)
	else:
	    self._icon = self.createcomponent('icon',
		    (), None,
		    Tkinter.Label, (interior,))
	    if iconpos not in 'nsew':
		raise ValueError, \
		    'bad iconpos option "%s":  should be n, s, e, or w' \
			% iconpos

	    if iconpos in 'nw':
		icon = 1
		message = 3
	    else:
		icon = 3
		message = 1

	    if iconpos in 'ns':
		# vertical layout
		self._icon.grid(column = 1, row = icon)
		self._message.grid(column = 1, row = message)
		interior.grid_rowconfigure(2, minsize = iconmargin)
		border_bottom = 4
	    else:
		# horizontal layout
		self._icon.grid(column = icon, row = 1)
		self._message.grid(column = message, row = 1)
		interior.grid_columnconfigure(2, minsize = iconmargin)
		border_right = 4

	interior.grid_columnconfigure(0, minsize = borderx)
	interior.grid_rowconfigure(0, minsize = bordery)
	interior.grid_columnconfigure(border_right, minsize = borderx)
	interior.grid_rowconfigure(border_bottom, minsize = bordery)


	# Check keywords and initialise options.
	self.initialiseoptions()



title = 'Pmw.LabeledWidget demonstration'

# Import Pmw from this directory tree.
import sys
sys.path[:0] = ['../../..']

import Tkinter
import Pmw

class Demo:
    def __init__(self, parent):

	# Create a frame to put the LabeledWidgets into
	frame = Tkinter.Frame(parent, background = 'grey90')
	frame.pack(fill = 'both', expand = 1)

	# Create and pack the LabeledWidgets.
	column = 0
	row = 0
	for pos in ('n', 'nw', 'wn', 'w'):
	    lw = Pmw.LabeledWidget(frame,
		    labelpos = pos,
		    label_text = pos + ' label')
	    lw.component('hull').configure(relief='sunken', borderwidth=2)
	    lw.grid(column=column, row=row, padx=10, pady=10)
	    cw = Tkinter.Button(lw.interior(), text='child\nsite')
	    cw.pack(padx=10, pady=10, expand='yes', fill='both')

	    # Get ready for next grid position.
	    column = column + 1
	    if column == 2:
	      column = 0
	      row = row + 1

######################################################################

# Create demo in root window for testing.
if __name__ == '__main__':
    root = Tkinter.Tk()
    Pmw.initialise(root, fontScheme = 'pmw1')
    root.title(title)

    widget = Demo(root)
    exitButton = Tkinter.Button(root, text = 'Exit', command = root.destroy)
    exitButton.pack()
    root.mainloop()



title = 'Pmw.PanedWidget demonstration (pane factory)'

# Import Pmw from this directory tree.
import sys
sys.path[:0] = ['../../..']

import Tkinter
import Pmw

class Demo:
    def __init__(self, parent):
	self.paneCount = 0

	# Create a "pane factory".
	label = Tkinter.Label(parent,
		pady = 10,
		text = 'Below is a simple "pane factory".\n' +
			'Drag the handle on the left\nto create new panes.')
	label.pack()
	self.factory = Pmw.PanedWidget(parent,
		orient='horizontal',
		command = self.resize,
		hull_borderwidth = 1,
		hull_relief = 'raised',
		hull_width=300, hull_height=200
                )
	self.factory.add('starter', size = 0.0)
	self.factory.add('main')
	button = Tkinter.Button(self.factory.pane('main'),
		text = 'Pane\n0')
	button.pack(expand = 1)
	self.factory.pack(expand = 1, fill = 'both')

    def resize(self, list):
	# Remove any panes less than 2 pixel wide.
	for i in range(len(list) - 1, 0, -1):
	    if list[i] < 2:
		self.factory.delete(i)

	# If the user has dragged the left hand handle, create a new pane.
	if list[0] > 1:
	    self.paneCount = self.paneCount + 1

	    # Add a button to the new pane.
	    name = self.factory.panes()[0]
	    text = 'Pane\n' + str(self.paneCount)
	    button = Tkinter.Button(self.factory.pane(name), text = text)
	    button.pack(expand = 1)

	    # Create a new starter pane.
	    name = 'Pane ' + str(self.paneCount)
	    self.factory.insert(name, size=0.0)

######################################################################

# Create demo in root window for testing.
if __name__ == '__main__':
    root = Tkinter.Tk()
    Pmw.initialise(root, fontScheme = 'pmw1')
    root.title(title)

    exitButton = Tkinter.Button(root, text = 'Exit', command = root.destroy)
    exitButton.pack(side = 'bottom')
    widget = Demo(root)
    root.mainloop()



import Pmw

class AboutDialog(Pmw.MessageDialog):
    # Window to display version and contact information.

    # Class members containing resettable 'default' values:
    _version = ''
    _copyright = ''
    _contact = ''

    def __init__(self, parent = None, **kw):

	# Define the megawidget options.
	INITOPT = Pmw.INITOPT
	optiondefs = (
	    ('applicationname',   '',          INITOPT),
	    ('iconpos',           'w',         None),
	    ('icon_bitmap',       'info',      None),
	    ('buttons',           ('Close',),  None),
	    ('defaultbutton',     0,           None),
	)
	self.defineoptions(kw, optiondefs)

	# Initialise the base class (after defining the options).
	Pmw.MessageDialog.__init__(self, parent)

	applicationname = self['applicationname']
        if not kw.has_key('title'):
            self.configure(title = 'About ' + applicationname)

        if not kw.has_key('message_text'):
            text = applicationname + '\n\n'
            if AboutDialog._version != '':
              text = text + 'Version ' + AboutDialog._version + '\n'
            if AboutDialog._copyright != '':
              text = text + AboutDialog._copyright + '\n\n'
            if AboutDialog._contact != '':
              text = text + AboutDialog._contact

            self.configure(message_text=text)

	# Check keywords and initialise options.
	self.initialiseoptions()

def aboutversion(value):
    AboutDialog._version = value

def aboutcopyright(value):
    AboutDialog._copyright = value

def aboutcontact(value):
    AboutDialog._contact = value



title = 'Demonstration of Pmw megawidget destruction'

# Import Pmw from this directory tree.
import sys
sys.path[:0] = ['../../..']

import Tkinter
import Pmw

class Demo:
    def __init__(self, parent):
	# Create and pack an EntryField.
	self.entryfield = Pmw.EntryField(parent,
	    command = self.execute,
            value = 'Press <Return> to destroy me',
	    entry_width = 30)
	self.entryfield.pack(fill='x', expand=1, padx=10, pady=5)

	self.entryfield.component('entry').focus_set()

    def execute(self):
	print 'Return pressed, destroying EntryField.'
	self.entryfield.destroy()

######################################################################

# Create demo in root window for testing.
if __name__ == '__main__':
    root = Tkinter.Tk()
    Pmw.initialise(root, fontScheme = 'pmw1')
    root.title(title)

    exitButton = Tkinter.Button(root, text = 'Exit', command = root.destroy)
    exitButton.pack(side = 'bottom')
    widget = Demo(root)
    root.mainloop()



title = 'Pmw.Counter demonstration'

# Import Pmw from this directory tree.
import sys
sys.path[:0] = ['../../..']

import string
import time
import Tkinter
import Pmw

class Demo:
    def __init__(self, parent):
	# Need to use long ints here because on the Macintosh the maximum size
	# of an integer is smaller than the value returned by time.time().
	now = (long(time.time()) / 300) * 300

	# Create the Counters.
	self._date = Pmw.Counter(parent,
		labelpos = 'w',
		label_text = 'Date (4-digit year):',
		entryfield_value =
                        time.strftime('%d/%m/%Y', time.localtime(now)),
	        entryfield_command = self.execute,
		entryfield_validate = {'validator' : 'date', 'format' : 'dmy'},
		datatype = {'counter' : 'date', 'format' : 'dmy', 'yyyy' : 1})

	self._isodate = Pmw.Counter(parent,
		labelpos = 'w',
		label_text = 'ISO-Date (4-digit year):',
		entryfield_value =
			time.strftime('%Y-%m-%d', time.localtime(now)),
		entryfield_command = self.execute,
		entryfield_validate = {'validator' : 'date', 'format' : 'ymd',
                        'separator' : '-' },
		datatype = {'counter' : 'date', 'format' : 'ymd', 'yyyy' : 1,
                        'separator' : '-' })

	self._time = Pmw.Counter(parent,
		labelpos = 'w',
		label_text = 'Time:',
		entryfield_value =
                        time.strftime('%H:%M:%S', time.localtime(now)),
	        entryfield_validate = {'validator' : 'time',
			'min' : '00:00:00', 'max' : '23:59:59',
			'minstrict' : 0, 'maxstrict' : 0},
		datatype = {'counter' : 'time', 'time24' : 1},
		increment=5*60)
	self._real = Pmw.Counter(parent,
		labelpos = 'w',
		label_text = 'Real (with comma):',
		entryfield_value = '1,5',
		datatype = {'counter' : 'real', 'separator' : ','},
	        entryfield_validate = {'validator' : 'real',
			'min' : '-2,0', 'max' : '5,0',
			'separator' : ','},
		increment = 0.1)
	self._custom = Pmw.Counter(parent,
		labelpos = 'w',
		label_text = 'Custom:',
		entryfield_value = specialword[:4],
		datatype = _custom_counter,
	        entryfield_validate = _custom_validate)
	self._int = Pmw.Counter(parent,
		labelpos = 'w',
		label_text = 'Vertical integer:',
		orient = 'vertical',
		entry_width = 2,
		entryfield_value = 50,
	        entryfield_validate = {'validator' : 'integer',
			'min' : 0, 'max' : 99}
	)

	counters = (self._date, self._isodate, self._time, self._real,
                self._custom)
	Pmw.alignlabels(counters)

	# Pack them all.
	for counter in counters:
	    counter.pack(fill='x', expand=1, padx=10, pady=5)
	self._int.pack(padx=10, pady=5)

    def execute(self):
	print 'Return pressed, value is', self._date.get()

specialword = 'Monti Python ik den Holie Grailen (Bok)'

def _custom_validate(text):
    if string.find(specialword, text) == 0:
	return 1
    else:
	return -1

def _custom_counter(text, factor, increment):
    # increment is ignored here.
    if string.find(specialword, text) == 0:
	length = len(text)
	if factor == 1:
	    if length >= len(specialword):
		raise ValueError, 'maximum length reached'
	    return specialword[:length + 1]
	else:
	    if length == 0:
		raise ValueError, 'empty string'
	    return specialword[:length - 1]
    else:
	raise ValueError, 'bad string ' + text

######################################################################

# Create demo in root window for testing.
if __name__ == '__main__':
    root = Tkinter.Tk()
    Pmw.initialise(root, fontScheme = 'pmw1')
    root.title(title)

    exitButton = Tkinter.Button(root, text = 'Exit', command = root.destroy)
    exitButton.pack(side = 'bottom')
    widget = Demo(root)
    root.mainloop()



title = 'Demonstration of how to create a megawidget'

# Import Pmw from this directory tree.
import sys
sys.path[:0] = ['../../..']

import Tkinter
import Pmw

class TextDisplay(Pmw.MegaWidget):

    # Demo Pmw megawidget.

    def __init__(self, parent = None, **kw):

	# Define the megawidget options.
	optiondefs = ()
	self.defineoptions(kw, optiondefs)

	# Initialise the base class (after defining the options).
	Pmw.MegaWidget.__init__(self, parent)

	# Create the components.
	interior = self.interior()

	self._text = self.createcomponent('text',
		(), None,
		Tkinter.Text, (interior,), state = 'disabled')
	self._text.pack(side='left', fill='both', expand='yes')

	self._scrollbar = self.createcomponent('scrollbar',
		(), None,
		Tkinter.Scrollbar, (interior,), command = self._text.yview)
	self._scrollbar.pack(side='right', fill='y')
	self._text.configure(yscrollcommand = self._scrollbar.set)

	# Check keywords and initialise options.
	self.initialiseoptions()

    def display(self, info):
	self._text.configure(state = 'normal')
	self._text.delete('1.0', 'end')
	self._text.insert('1.0', info)
	self._text.configure(state = 'disabled')

    def append(self, info):
	self._text.configure(state = 'normal')
	self._text.insert('end', info)
	self._text.configure(state = 'disabled')

class Demo:
    def __init__(self, parent):
	# Create and pack the megawidget.
	text = TextDisplay(parent,
		text_background = 'aliceblue',
		text_width = 40,
		text_height = 10,
		text_wrap = 'none',
	)
	text.pack(fill = 'both', expand = 1)
	text.display('This is an example of a simple Pmw megawidget.\n\n' +
		'Public attributes of the Tkinter module:\n\n')
	for name in dir(Tkinter):
	    if name[0] != '_':
		text.append('    ' + name + '\n')

######################################################################

# Create demo in root window for testing.
if __name__ == '__main__':
    root = Tkinter.Tk()
    Pmw.initialise(root, fontScheme = 'pmw1')
    root.title(title)

    exitButton = Tkinter.Button(root, text = 'Exit', command = root.destroy)
    exitButton.pack(side = 'bottom')
    widget = Demo(root)
    root.mainloop()



# Class to display messages in an information line.

import string
import Tkinter
import Pmw

class MessageBar(Pmw.MegaWidget):
    def __init__(self, parent = None, **kw):

	# Define the megawidget options.
	INITOPT = Pmw.INITOPT
	defaultMessageTypes = {
			   # (priority, showtime, bells, logmessage)
	    'systemerror'  : (5, 10, 2, 1),
	    'usererror'    : (4, 5, 1, 0),
	    'busy'         : (3, 0, 0, 0),
	    'systemevent'  : (2, 5, 0, 0),
	    'userevent'    : (2, 5, 0, 0),
	    'help'         : (1, 5, 0, 0),
	    'state'        : (0, 0, 0, 0),
	}
	optiondefs = (
	    ('labelmargin',    0,                     INITOPT),
	    ('labelpos',       None,                  INITOPT),
	    ('messagetypes',   defaultMessageTypes,   INITOPT),
	    ('silent',         0,                     None),
	)
	self.defineoptions(kw, optiondefs)

	# Initialise the base class (after defining the options).
	Pmw.MegaWidget.__init__(self, parent)

	# Create the components.
	interior = self.interior()
	self._messageBarEntry = self.createcomponent('entry',
		(), None,
		Tkinter.Entry, (interior,), state = 'disabled')
	self._messageBarEntry.grid(column=2, row=2, sticky='nsew')
	interior.grid_columnconfigure(2, weight=1)
	interior.grid_rowconfigure(2, weight=1)

	self.createlabel(interior)

	# Initialise instance variables.
	self._numPriorities = 0
	for info in self['messagetypes'].values():
	    if self._numPriorities < info[0]:
		self._numPriorities = info[0]

	self._numPriorities = self._numPriorities + 1
	self._timer = [None] * self._numPriorities
	self._messagetext = [''] * self._numPriorities
	self._activemessage = [0] * self._numPriorities

	# Check keywords and initialise options.
	self.initialiseoptions()

    def destroy(self):
	for timerId in self._timer:
	    if timerId is not None:
		self.after_cancel(timerId)
	self._timer = [None] * self._numPriorities
	Pmw.MegaWidget.destroy(self)

    def message(self, type, text):
        # Display a message in the message bar.

	(priority, showtime, bells, logmessage) = self['messagetypes'][type]

	if not self['silent']:
	    for i in range(bells):
		if i != 0:
		    self.after(100)
		self.bell()

	self._activemessage[priority] = 1
	if text is None:
	    text = ''
	self._messagetext[priority] = string.replace(text, '\n', ' ')
	self._redisplayInfoMessage()

	if logmessage:
	    # Should log this text to a text widget.
	    pass

	if showtime > 0:
	    if self._timer[priority] is not None:
		self.after_cancel(self._timer[priority])

	    # Define a callback to clear this message after a time.
	    def _clearmessage(self=self, priority=priority):
		self._clearActivemessage(priority)

	    mseconds = int(showtime * 1000)
	    self._timer[priority] = self.after(mseconds, _clearmessage)

    def helpmessage(self, text):
        if text is None:
            self.resetmessages('help')
        else:
            self.message('help', text)

    def resetmessages(self, type):
	priority = self['messagetypes'][type][0]
	self._clearActivemessage(priority)
	for messagetype, info in self['messagetypes'].items():
	    thisPriority = info[0]
	    showtime = info[1]
	    if thisPriority < priority and showtime != 0:
		self._clearActivemessage(thisPriority)

    def _clearActivemessage(self, priority):
	self._activemessage[priority] = 0
	if self._timer[priority] is not None:
	    self.after_cancel(self._timer[priority])
	    self._timer[priority] = None
	self._redisplayInfoMessage()

    def _redisplayInfoMessage(self):
	text = ''
        for priority in range(self._numPriorities - 1, -1, -1):
	    if self._activemessage[priority]:
		text = self._messagetext[priority]
	        break
	self._messageBarEntry.configure(state = 'normal')
	self._messageBarEntry.delete(0, 'end')
	self._messageBarEntry.insert('end', text)
	self._messageBarEntry.configure(state = 'disabled')

Pmw.forwardmethods(MessageBar, Tkinter.Entry, '_messageBarEntry')



# Based on iwidgets2.2.0/scrolledtext.itk code.   

import Tkinter
import Pmw

class ScrolledText(Pmw.MegaWidget):
    def __init__(self, parent = None, **kw):

	# Define the megawidget options.
	INITOPT = Pmw.INITOPT
	optiondefs = (
	    ('borderframe',    0,            INITOPT),
	    ('columnheader',   0,            INITOPT),
	    ('hscrollmode',    'dynamic',    self._hscrollMode),
	    ('labelmargin',    0,            INITOPT),
	    ('labelpos',       None,         INITOPT),
	    ('rowcolumnheader',0,            INITOPT),
	    ('rowheader',      0,            INITOPT),
	    ('scrollmargin',   2,            INITOPT),
	    ('usehullsize',    0,            INITOPT),
	    ('vscrollmode',    'dynamic',    self._vscrollMode),
	)
	self.defineoptions(kw, optiondefs)

	# Initialise the base class (after defining the options).
	Pmw.MegaWidget.__init__(self, parent)

	# Create the components.
	interior = self.interior()

	if self['usehullsize']:
	    interior.grid_propagate(0)

	if self['borderframe']:
	    # Create a frame widget to act as the border of the text 
	    # widget.  Later, pack the text widget so that it fills
	    # the frame.  This avoids a problem in Tk, where window
	    # items in a text widget may overlap the border of the
	    # text widget.
	    self._borderframe = self.createcomponent('borderframe',
		    (), None,
		    Tkinter.Frame, (interior,),
		    relief = 'sunken',
		    borderwidth = 2,
	    )
	    self._borderframe.grid(row = 4, column = 4, sticky = 'news')

	    # Create the text widget.
	    self._textbox = self.createcomponent('text',
		    (), None,
		    Tkinter.Text, (self._borderframe,),
		    highlightthickness = 0,
		    borderwidth = 0,
	    )
	    self._textbox.pack(fill = 'both', expand = 1)

            bw = self._borderframe.cget('borderwidth'),
            ht = self._borderframe.cget('highlightthickness'),
	else:
	    # Create the text widget.
	    self._textbox = self.createcomponent('text',
		    (), None,
		    Tkinter.Text, (interior,),
	    )
	    self._textbox.grid(row = 4, column = 4, sticky = 'news')

            bw = self._textbox.cget('borderwidth'),
            ht = self._textbox.cget('highlightthickness'),

        # Create the header text widgets
        if self['columnheader']:
            self._columnheader = self.createcomponent('columnheader',
                    (), 'Header',
                    Tkinter.Text, (interior,),
                    height=1,
                    wrap='none',
                    borderwidth = bw,
                    highlightthickness = ht,
            )
            self._columnheader.grid(row = 2, column = 4, sticky = 'ew')
            self._columnheader.configure(
                    xscrollcommand = self._columnheaderscrolled)

        if self['rowheader']:
            self._rowheader = self.createcomponent('rowheader',
                    (), 'Header',
                    Tkinter.Text, (interior,),
                    wrap='none',
                    borderwidth = bw,
                    highlightthickness = ht,
            )
            self._rowheader.grid(row = 4, column = 2, sticky = 'ns')
            self._rowheader.configure(
                    yscrollcommand = self._rowheaderscrolled)

        if self['rowcolumnheader']:
            self._rowcolumnheader = self.createcomponent('rowcolumnheader',
                    (), 'Header',
                    Tkinter.Text, (interior,),
                    height=1,
                    wrap='none',
                    borderwidth = bw,
                    highlightthickness = ht,
            )
            self._rowcolumnheader.grid(row = 2, column = 2, sticky = 'nsew')

	interior.grid_rowconfigure(4, weight = 1, minsize = 0)
	interior.grid_columnconfigure(4, weight = 1, minsize = 0)

	# Create the horizontal scrollbar
	self._horizScrollbar = self.createcomponent('horizscrollbar',
		(), 'Scrollbar',
		Tkinter.Scrollbar, (interior,),
	        orient='horizontal',
		command=self._textbox.xview
	)

	# Create the vertical scrollbar
	self._vertScrollbar = self.createcomponent('vertscrollbar',
		(), 'Scrollbar',
		Tkinter.Scrollbar, (interior,),
		orient='vertical',
		command=self._textbox.yview
	)

	self.createlabel(interior, childCols = 5, childRows = 5)

	# Initialise instance variables.
	self._horizScrollbarOn = 0
	self._vertScrollbarOn = 0
	self.scrollTimer = None
        self._scrollRecurse = 0
	self._horizScrollbarNeeded = 0
	self._vertScrollbarNeeded = 0
	self._textWidth = None

        # These four variables avoid an infinite loop caused by the
        # row or column header's scrollcommand causing the main text
        # widget's scrollcommand to be called and vice versa.
	self._textboxLastX = None
	self._textboxLastY = None
	self._columnheaderLastX = None
	self._rowheaderLastY = None

	# Check keywords and initialise options.
	self.initialiseoptions()

    def destroy(self):
	if self.scrollTimer is not None:
	    self.after_cancel(self.scrollTimer)
	    self.scrollTimer = None
	Pmw.MegaWidget.destroy(self)

    # ======================================================================

    # Public methods.

    def clear(self):
	self.settext('')

    def importfile(self, fileName, where = 'end'):
	file = open(fileName, 'r')
	self._textbox.insert(where, file.read())
	file.close()

    def exportfile(self, fileName):
	file = open(fileName, 'w')
	file.write(self._textbox.get('1.0', 'end'))
	file.close()

    def settext(self, text):
	disabled = (self._textbox.cget('state') == 'disabled')
	if disabled:
	    self._textbox.configure(state='normal')
	self._textbox.delete('0.0', 'end')
	self._textbox.insert('end', text)
	if disabled:
	    self._textbox.configure(state='disabled')

    # Override Tkinter.Text get method, so that if it is called with
    # no arguments, return all text (consistent with other widgets).
    def get(self, first=None, last=None):
	if first is None:
	    return self._textbox.get('1.0', 'end')
	else:
	    return self._textbox.get(first, last)

    def getvalue(self):
        return self.get()

    def setvalue(self, text):
        return self.settext(text)

    # ======================================================================

    # Configuration methods.

    def _hscrollMode(self):
	# The horizontal scroll mode has been configured.

	mode = self['hscrollmode']

	if mode == 'static':
	    if not self._horizScrollbarOn:
		self._toggleHorizScrollbar()
	elif mode == 'dynamic':
	    if self._horizScrollbarNeeded != self._horizScrollbarOn:
		self._toggleHorizScrollbar()
	elif mode == 'none':
	    if self._horizScrollbarOn:
		self._toggleHorizScrollbar()
	else:
	    message = 'bad hscrollmode option "%s": should be static, dynamic, or none' % mode
	    raise ValueError, message

        self._configureScrollCommands()

    def _vscrollMode(self):
	# The vertical scroll mode has been configured.

	mode = self['vscrollmode']

	if mode == 'static':
	    if not self._vertScrollbarOn:
		self._toggleVertScrollbar()
	elif mode == 'dynamic':
	    if self._vertScrollbarNeeded != self._vertScrollbarOn:
		self._toggleVertScrollbar()
	elif mode == 'none':
	    if self._vertScrollbarOn:
		self._toggleVertScrollbar()
	else:
	    message = 'bad vscrollmode option "%s": should be static, dynamic, or none' % mode
	    raise ValueError, message

        self._configureScrollCommands()

    # ======================================================================

    # Private methods.

    def _configureScrollCommands(self):
        # If both scrollmodes are not dynamic we can save a lot of
        # time by not having to create an idle job to handle the
        # scroll commands.

        # Clean up previous scroll commands to prevent memory leak.
        tclCommandName = self._textbox.cget('xscrollcommand')
        if tclCommandName != '':   
            self._textbox.deletecommand(tclCommandName)
        tclCommandName = self._textbox.cget('yscrollcommand')
        if tclCommandName != '':   
            self._textbox.deletecommand(tclCommandName)

	if self['hscrollmode'] == self['vscrollmode'] == 'dynamic':
            self._textbox.configure(
                    xscrollcommand=self._scrollBothLater,
                    yscrollcommand=self._scrollBothLater
            )
        else:
            self._textbox.configure(
                    xscrollcommand=self._scrollXNow,
                    yscrollcommand=self._scrollYNow
            )

    def _scrollXNow(self, first, last):
        self._horizScrollbar.set(first, last)
        self._horizScrollbarNeeded = ((first, last) != ('0', '1'))

        # This code is the same as in _scrollBothNow.  Keep it that way.
        if self['hscrollmode'] == 'dynamic':
            currentWidth = self._textbox.winfo_width()
            if self._horizScrollbarNeeded != self._horizScrollbarOn:
                if self._horizScrollbarNeeded or \
                        self._textWidth != currentWidth:
                    self._toggleHorizScrollbar()
            self._textWidth = currentWidth

        if self['columnheader']:
	    if self._columnheaderLastX != first:
		self._columnheaderLastX = first
		self._columnheader.xview('moveto', first)

    def _scrollYNow(self, first, last):
        if first == '0' and last == '0':
            return
        self._vertScrollbar.set(first, last)
        self._vertScrollbarNeeded = ((first, last) != ('0', '1'))

        if self['vscrollmode'] == 'dynamic':
            if self._vertScrollbarNeeded != self._vertScrollbarOn:
                self._toggleVertScrollbar()

        if self['rowheader']:
	    if self._rowheaderLastY != first:
		self._rowheaderLastY = first
		self._rowheader.yview('moveto', first)

    def _scrollBothLater(self, first, last):
	# Called by the text widget to set the horizontal or vertical
	# scrollbar when it has scrolled or changed size or contents.

	if self.scrollTimer is None:
	    self.scrollTimer = self.after_idle(self._scrollBothNow)

    def _scrollBothNow(self):
        # This performs the function of _scrollXNow and _scrollYNow.
        # If one is changed, the other should be updated to match.
	self.scrollTimer = None

        # Call update_idletasks to make sure that the containing frame
        # has been resized before we attempt to set the scrollbars. 
        # Otherwise the scrollbars may be mapped/unmapped continuously.
        self._scrollRecurse = self._scrollRecurse + 1
        self.update_idletasks()
        self._scrollRecurse = self._scrollRecurse - 1
        if self._scrollRecurse != 0:
            return

	xview = self._textbox.xview()
	yview = self._textbox.yview()

	# The text widget returns a yview of (0.0, 0.0) just after it
	# has been created. Ignore this.
	if yview == (0.0, 0.0):
	    return

        if self['columnheader']:
	    if self._columnheaderLastX != xview[0]:
		self._columnheaderLastX = xview[0]
		self._columnheader.xview('moveto', xview[0])
        if self['rowheader']:
	    if self._rowheaderLastY != yview[0]:
		self._rowheaderLastY = yview[0]
		self._rowheader.yview('moveto', yview[0])

	self._horizScrollbar.set(xview[0], xview[1])
	self._vertScrollbar.set(yview[0], yview[1])

	self._horizScrollbarNeeded = (xview != (0.0, 1.0))
	self._vertScrollbarNeeded = (yview != (0.0, 1.0))

	# If both horizontal and vertical scrollmodes are dynamic and
	# currently only one scrollbar is mapped and both should be
	# toggled, then unmap the mapped scrollbar.  This prevents a
	# continuous mapping and unmapping of the scrollbars. 
	if (self['hscrollmode'] == self['vscrollmode'] == 'dynamic' and
		self._horizScrollbarNeeded != self._horizScrollbarOn and
		self._vertScrollbarNeeded != self._vertScrollbarOn and
		self._vertScrollbarOn != self._horizScrollbarOn):
	    if self._horizScrollbarOn:
		self._toggleHorizScrollbar()
	    else:
		self._toggleVertScrollbar()
	    return

	if self['hscrollmode'] == 'dynamic':

	    # The following test is done to prevent continuous
	    # mapping and unmapping of the horizontal scrollbar. 
	    # This may occur when some event (scrolling, resizing
	    # or text changes) modifies the displayed text such
	    # that the bottom line in the window is the longest
	    # line displayed.  If this causes the horizontal
	    # scrollbar to be mapped, the scrollbar may "cover up"
	    # the bottom line, which would mean that the scrollbar
	    # is no longer required.  If the scrollbar is then
	    # unmapped, the bottom line will then become visible
	    # again, which would cause the scrollbar to be mapped
	    # again, and so on...
	    #
	    # The idea is that, if the width of the text widget
	    # has not changed and the scrollbar is currently
	    # mapped, then do not unmap the scrollbar even if it
	    # is no longer required.  This means that, during
	    # normal scrolling of the text, once the horizontal
	    # scrollbar has been mapped it will not be unmapped
	    # (until the width of the text widget changes).

	    currentWidth = self._textbox.winfo_width()
	    if self._horizScrollbarNeeded != self._horizScrollbarOn:
		if self._horizScrollbarNeeded or \
			self._textWidth != currentWidth:
		    self._toggleHorizScrollbar()
	    self._textWidth = currentWidth

	if self['vscrollmode'] == 'dynamic':
	    if self._vertScrollbarNeeded != self._vertScrollbarOn:
		self._toggleVertScrollbar()

    def _columnheaderscrolled(self, first, last):
	if self._textboxLastX != first:
	    self._textboxLastX = first
	    self._textbox.xview('moveto', first)

    def _rowheaderscrolled(self, first, last):
	if self._textboxLastY != first:
	    self._textboxLastY = first
	    self._textbox.yview('moveto', first)

    def _toggleHorizScrollbar(self):

	self._horizScrollbarOn = not self._horizScrollbarOn

	interior = self.interior()
	if self._horizScrollbarOn:
	    self._horizScrollbar.grid(row = 6, column = 4, sticky = 'news')
	    interior.grid_rowconfigure(5, minsize = self['scrollmargin'])
	else:
	    self._horizScrollbar.grid_forget()
	    interior.grid_rowconfigure(5, minsize = 0)

    def _toggleVertScrollbar(self):

	self._vertScrollbarOn = not self._vertScrollbarOn

	interior = self.interior()
	if self._vertScrollbarOn:
	    self._vertScrollbar.grid(row = 4, column = 6, sticky = 'news')
	    interior.grid_columnconfigure(5, minsize = self['scrollmargin'])
	else:
	    self._vertScrollbar.grid_forget()
	    interior.grid_columnconfigure(5, minsize = 0)

    # Need to explicitly forward this to override the stupid
    # (grid_)bbox method inherited from Tkinter.Frame.Grid.
    def bbox(self, index):
	return self._textbox.bbox(index)

Pmw.forwardmethods(ScrolledText, Tkinter.Text, '_textbox')



# Authors: Joe VanAndel and Greg McFarlane

import string
import sys
import time
import Tkinter
import Pmw

class TimeCounter(Pmw.MegaWidget):
    """Up-down counter

    A TimeCounter is a single-line entry widget with Up and Down arrows
    which increment and decrement the Time value in the entry.  
    """

    def __init__(self, parent = None, **kw):

	# Define the megawidget options.
	INITOPT = Pmw.INITOPT
	optiondefs = (
	    ('autorepeat',    1,    None),
	    ('buttonaspect',  1.0,  INITOPT),
	    ('command',       None, None),
	    ('initwait',      300,  None),
	    ('labelmargin',   0,    INITOPT),
	    ('labelpos',      None, INITOPT),
	    ('max',           None, self._max),
	    ('min',           None, self._min),
	    ('padx',          0,    INITOPT),
	    ('pady',          0,    INITOPT),
	    ('repeatrate',    50,   None),
	    ('value',         None, INITOPT),
	)
	self.defineoptions(kw, optiondefs)

	# Initialise the base class (after defining the options).
	Pmw.MegaWidget.__init__(self, parent)

    	self.arrowDirection = {}
	self._flag = 'stopped'
	self._timerId = None

	self._createComponents(kw)

	value = self['value']
	if value is None:
	    now = time.time()
	    value = time.strftime('%H:%M:%S', time.localtime(now))
    	self.setvalue(value)

	# Check keywords and initialise options.
	self.initialiseoptions()

    def _createComponents(self, kw):

	# Create the components.
	interior = self.interior()

	# If there is no label, put the arrows and the entry directly
	# into the interior, otherwise create a frame for them.  In
	# either case the border around the arrows and the entry will
	# be raised (but not around the label).
	if self['labelpos'] is None:
	    frame = interior
            if not kw.has_key('hull_relief'):
                frame.configure(relief = 'raised')
            if not kw.has_key('hull_borderwidth'):
                frame.configure(borderwidth = 1)
	else:
	    frame = self.createcomponent('frame',
		    (), None,
		    Tkinter.Frame, (interior,),
                    relief = 'raised', borderwidth = 1)
	    frame.grid(column=2, row=2, sticky='nsew')
	    interior.grid_columnconfigure(2, weight=1)
	    interior.grid_rowconfigure(2, weight=1)

	# Create the down arrow buttons.

	# Create the hour down arrow.
	self._downHourArrowBtn = self.createcomponent('downhourarrow',
		(), 'Arrow',
		Tkinter.Canvas, (frame,),
		width = 16, height = 16, relief = 'raised', borderwidth = 2)
    	self.arrowDirection[self._downHourArrowBtn] = 'down'
	self._downHourArrowBtn.grid(column = 0, row = 2)

	# Create the minute down arrow.
	self._downMinuteArrowBtn = self.createcomponent('downminutearrow',
		(), 'Arrow',
		Tkinter.Canvas, (frame,),
		width = 16, height = 16, relief = 'raised', borderwidth = 2)
    	self.arrowDirection[self._downMinuteArrowBtn] = 'down'
	self._downMinuteArrowBtn.grid(column = 1, row = 2)

	# Create the second down arrow.
	self._downSecondArrowBtn = self.createcomponent('downsecondarrow',
		(), 'Arrow',
		Tkinter.Canvas, (frame,),
		width = 16, height = 16, relief = 'raised', borderwidth = 2)
    	self.arrowDirection[self._downSecondArrowBtn] = 'down'
	self._downSecondArrowBtn.grid(column = 2, row = 2)

	# Create the entry fields.

	# Create the hour entry field.
	self._hourCounterEntry = self.createcomponent('hourentryfield',
		(('hourentry', 'hourentryfield_entry'),), None,
		Pmw.EntryField, (frame,), validate='integer', entry_width = 2)
	self._hourCounterEntry.grid(column = 0, row = 1, sticky = 'news')

	# Create the minute entry field.
	self._minuteCounterEntry = self.createcomponent('minuteentryfield',
		(('minuteentry', 'minuteentryfield_entry'),), None,
		Pmw.EntryField, (frame,), validate='integer', entry_width = 2)
	self._minuteCounterEntry.grid(column = 1, row = 1, sticky = 'news')

	# Create the second entry field.
	self._secondCounterEntry = self.createcomponent('secondentryfield',
		(('secondentry', 'secondentryfield_entry'),), None,
		Pmw.EntryField, (frame,), validate='integer', entry_width = 2)
	self._secondCounterEntry.grid(column = 2, row = 1, sticky = 'news')

	# Create the up arrow buttons.

	# Create the hour up arrow.
	self._upHourArrowBtn = self.createcomponent('uphourarrow',
		(), 'Arrow',
		Tkinter.Canvas, (frame,),
		width = 16, height = 16, relief = 'raised', borderwidth = 2)
    	self.arrowDirection[self._upHourArrowBtn] = 'up'
	self._upHourArrowBtn.grid(column = 0, row = 0)

	# Create the minute up arrow.
	self._upMinuteArrowBtn = self.createcomponent('upminutearrow',
		(), 'Arrow',
		Tkinter.Canvas, (frame,),
		width = 16, height = 16, relief = 'raised', borderwidth = 2)
    	self.arrowDirection[self._upMinuteArrowBtn] = 'up'
	self._upMinuteArrowBtn.grid(column = 1, row = 0)

	# Create the second up arrow.
	self._upSecondArrowBtn = self.createcomponent('upsecondarrow',
		(), 'Arrow',
		Tkinter.Canvas, (frame,),
		width = 16, height = 16, relief = 'raised', borderwidth = 2)
    	self.arrowDirection[self._upSecondArrowBtn] = 'up'
	self._upSecondArrowBtn.grid(column = 2, row = 0)

	# Make it resize nicely.
	padx = self['padx']
	pady = self['pady']
	for col in range(3):
	    frame.grid_columnconfigure(col, weight = 1, pad = padx)
	frame.grid_rowconfigure(0, pad = pady)
	frame.grid_rowconfigure(2, pad = pady)

	frame.grid_rowconfigure(1, weight = 1)

	# Create the label.
	self.createlabel(interior)

	# Set bindings.

	# Up hour
	self._upHourArrowBtn.bind('<Configure>', 
		lambda  event, s=self,button=self._upHourArrowBtn: 
		s._drawArrow(button, 'up'))

	self._upHourArrowBtn.bind('<1>', 
    	    	lambda event, s=self,button=self._upHourArrowBtn: 
		s._countUp(button, 3600))

	self._upHourArrowBtn.bind('<Any-ButtonRelease-1>', 
		lambda event, s=self, button=self._upHourArrowBtn:
		s._stopUpDown(button))

	# Up minute
	self._upMinuteArrowBtn.bind('<Configure>', 
		lambda  event, s=self,button=self._upMinuteArrowBtn: 
		s._drawArrow(button, 'up'))
	    

	self._upMinuteArrowBtn.bind('<1>', 
    	    	lambda event, s=self,button=self._upMinuteArrowBtn: 
		s._countUp(button, 60))

	self._upMinuteArrowBtn.bind('<Any-ButtonRelease-1>', 
		lambda event, s=self, button=self._upMinuteArrowBtn:
		s._stopUpDown(button))

	# Up second
	self._upSecondArrowBtn.bind('<Configure>', 
		lambda  event, s=self,button=self._upSecondArrowBtn: 
		s._drawArrow(button, 'up'))
	    

	self._upSecondArrowBtn.bind('<1>', 
    	    	lambda event, s=self,button=self._upSecondArrowBtn: 
		s._countUp(button, 1))

	self._upSecondArrowBtn.bind('<Any-ButtonRelease-1>', 
		lambda event, s=self, button=self._upSecondArrowBtn:
		s._stopUpDown(button))

	# Down hour
	self._downHourArrowBtn.bind('<Configure>', 
		lambda  event, s=self,button=self._downHourArrowBtn: 
		s._drawArrow(button, 'down'))

	self._downHourArrowBtn.bind('<1>', 
    	    	lambda event, s=self,button=self._downHourArrowBtn: 
		s._countDown(button, 3600))
	self._downHourArrowBtn.bind('<Any-ButtonRelease-1>', 
		lambda event, s=self, button=self._downHourArrowBtn:
		s._stopUpDown(button))


	# Down minute
	self._downMinuteArrowBtn.bind('<Configure>', 
		lambda  event, s=self,button=self._downMinuteArrowBtn: 
		s._drawArrow(button, 'down'))

	self._downMinuteArrowBtn.bind('<1>', 
    	    	lambda event, s=self,button=self._downMinuteArrowBtn:
                s._countDown(button, 60))
	self._downMinuteArrowBtn.bind('<Any-ButtonRelease-1>', 
		lambda event, s=self, button=self._downMinuteArrowBtn:
		s._stopUpDown(button))

	# Down second
	self._downSecondArrowBtn.bind('<Configure>', 
		lambda  event, s=self,button=self._downSecondArrowBtn: 
		s._drawArrow(button, 'down'))

	self._downSecondArrowBtn.bind('<1>', 
    	    	lambda event, s=self, button=self._downSecondArrowBtn: 
		s._countDown(button,1))
	self._downSecondArrowBtn.bind('<Any-ButtonRelease-1>', 
		lambda event, s=self, button=self._downSecondArrowBtn:
		s._stopUpDown(button))

	self._hourCounterEntry.component('entry').bind(
                '<Return>', self._invoke)
	self._minuteCounterEntry.component('entry').bind(
        	'<Return>', self._invoke)
	self._secondCounterEntry.component('entry').bind(
        	'<Return>', self._invoke)

	self._hourCounterEntry.bind('<Configure>', self._resizeArrow)
	self._minuteCounterEntry.bind('<Configure>', self._resizeArrow)
	self._secondCounterEntry.bind('<Configure>', self._resizeArrow)

    def _drawArrow(self, arrow, direction):
        Pmw.drawarrow(arrow, self['hourentry_foreground'], direction, 'arrow')

    def _resizeArrow(self, event = None):
	for btn in (self._upHourArrowBtn, self._upMinuteArrowBtn,
		self._upSecondArrowBtn,
		self._downHourArrowBtn,
		self._downMinuteArrowBtn, self._downSecondArrowBtn):
	    bw = (string.atoi(btn['borderwidth']) +
		    string.atoi(btn['highlightthickness']))
	    newHeight = self._hourCounterEntry.winfo_reqheight() - 2 * bw
	    newWidth = int(newHeight * self['buttonaspect'])
	    btn.configure(width=newWidth, height=newHeight)
	    self._drawArrow(btn, self.arrowDirection[btn])

    def _min(self):
	min = self['min']
        if min is None:
	    self._minVal = 0
	else:
	    self._minVal = Pmw.timestringtoseconds(min)

    def _max(self):
	max = self['max']
	if max is None:
	    self._maxVal = None
	else:
	    self._maxVal = Pmw.timestringtoseconds(max)

    def getvalue(self):
        return self.getstring()

    def setvalue(self, text):
        list = string.split(text, ':')
	if len(list) != 3:
	    raise ValueError, 'invalid value: ' + text

	self._hour = string.atoi(list[0])
	self._minute = string.atoi(list[1])
	self._second = string.atoi(list[2]) 

    	self._setHMS()

    def getstring(self):
    	return '%02d:%02d:%02d' % (self._hour, self._minute, self._second)

    def getint(self):
    	return self._hour * 3600 + self._minute * 60 + self._second

    def _countUp(self, button, increment):
	self._relief = self._upHourArrowBtn.cget('relief')
	button.configure(relief='sunken')
	self._count(1, 'start', increment)

    def _countDown(self, button, increment):

	self._relief = self._downHourArrowBtn.cget('relief')
	button.configure(relief='sunken')
	self._count(-1, 'start', increment)

    def increment(self, seconds = 1):
	self._count(1, 'force', seconds)

    def decrement(self, seconds = 1):
	self._count(-1, 'force', seconds)

    def _count(self, factor, newFlag = None, increment = 1):
	if newFlag != 'force':
	  if newFlag is not None:
	    self._flag = newFlag

	  if self._flag == 'stopped':
	    return

	value = (string.atoi(self._hourCounterEntry.get()) *3600) + \
	      (string.atoi(self._minuteCounterEntry.get()) *60) + \
	      string.atoi(self._secondCounterEntry.get()) + \
	      factor * increment
	min = self._minVal
	max = self._maxVal
	if value < min:
	  value = min
	if max is not None and value > max:
	  value = max

	self._hour = value /3600
	self._minute = (value - (self._hour*3600)) / 60
	self._second = value - (self._hour*3600) - (self._minute*60)
	self._setHMS()

	if newFlag != 'force':
	  if self['autorepeat']:
	    if self._flag == 'start':
	      delay = self['initwait']
	      self._flag = 'running'
	    else:
	      delay = self['repeatrate']
	    self._timerId = self.after(
		delay, lambda self=self, factor=factor,increment=increment: 
		  self._count(factor,'running', increment))

    def _setHMS(self):
        self._hourCounterEntry.setentry('%02d' % self._hour)
        self._minuteCounterEntry.setentry('%02d' % self._minute)
        self._secondCounterEntry.setentry('%02d' % self._second)

    def _stopUpDown(self, button):
        if self._timerId is not None:
            self.after_cancel(self._timerId)
	    self._timerId = None
        button.configure(relief=self._relief)
        self._flag = 'stopped'

    def _invoke(self, event):
        cmd = self['command']
        if callable(cmd):
	    cmd()

    def invoke(self):
        cmd = self['command']
        if callable(cmd):
	    return cmd()

    def destroy(self):
        if self._timerId is not None:
            self.after_cancel(self._timerId)
	    self._timerId = None
        Pmw.MegaWidget.destroy(self)



title = 'Pmw.AboutDialog demonstration'

# Import Pmw from this directory tree.
import sys
sys.path[:0] = ['../../..']

import Tkinter
import Pmw

class Demo:
    def __init__(self, parent):
	# Create dialog.
	Pmw.aboutversion('9.9')
	Pmw.aboutcopyright('Copyright My Company 1999\nAll rights reserved')
	Pmw.aboutcontact(
	    'For information about this application contact:\n' +
	    '  My Help Desk\n' +
	    '  Phone: +61 2 9876 5432\n' +
	    '  email: help@my.company.com.au'
	)
	self.about = Pmw.AboutDialog(parent, applicationname = 'My Application')
	self.about.withdraw()

	# Create button to launch the dialog.
	w = Tkinter.Button(parent, text = 'Show about dialog',
	        command = self.execute)
	w.pack(padx = 8, pady = 8)

    def execute(self):
	self.about.show()

######################################################################

# Create demo in root window for testing.
if __name__ == '__main__':
    root = Tkinter.Tk()
    Pmw.initialise(root, fontScheme = 'pmw1')
    root.title(title)

    exitButton = Tkinter.Button(root, text = 'Exit', command = root.destroy)
    exitButton.pack(side = 'bottom')
    widget = Demo(root)
    root.mainloop()



# Based on iwidgets2.2.0/promptdialog.itk code.

import Pmw

# A Dialog with an entryfield

class PromptDialog(Pmw.Dialog):
    def __init__(self, parent = None, **kw):
	# Define the megawidget options.
	INITOPT = Pmw.INITOPT
	optiondefs = (
	    ('borderx',     20,    INITOPT),
	    ('bordery',     20,    INITOPT),
	)
	self.defineoptions(kw, optiondefs)

	# Initialise the base class (after defining the options).
	Pmw.Dialog.__init__(self, parent)

	# Create the components.
	interior = self.interior()
	aliases = (
	    ('entry', 'entryfield_entry'),
	    ('label', 'entryfield_label'),
	)
	self._promptDialogEntry = self.createcomponent('entryfield',
		aliases, None,
		Pmw.EntryField, (interior,))
	self._promptDialogEntry.pack(fill='x', expand=1,
		padx = self['borderx'], pady = self['bordery'])
	
        if not kw.has_key('activatecommand'):
            # Whenever this dialog is activated, set the focus to the
            # EntryField's entry widget.
            tkentry = self.component('entry')
            self.configure(activatecommand = tkentry.focus_set)

	# Check keywords and initialise options.
	self.initialiseoptions()

    # Supply aliases to some of the entry component methods.
    def insertentry(self, index, text):
	self._promptDialogEntry.insert(index, text)

    def deleteentry(self, first, last=None):
	self._promptDialogEntry.delete(first, last)

    def indexentry(self, index):
	return self._promptDialogEntry.index(index)

Pmw.forwardmethods(PromptDialog, Pmw.EntryField, '_promptDialogEntry')



title = 'Pmw.Dialog demonstration'

# Import Pmw from this directory tree.
import sys
sys.path[:0] = ['../../..']

import Tkinter
import Pmw

class Demo:
    def __init__(self, parent):
	# Create two buttons to launch the dialog.
	w = Tkinter.Button(parent, text = 'Show application modal dialog',
	        command = self.showAppModal)
	w.pack(padx = 8, pady = 8)

	w = Tkinter.Button(parent, text = 'Show global modal dialog',
	        command = self.showGlobalModal)
	w.pack(padx = 8, pady = 8)

	w = Tkinter.Button(parent, text = 'Show dialog with "no grab"',
	        command = self.showDialogNoGrab)
	w.pack(padx = 8, pady = 8)

	w = Tkinter.Button(parent, text =
                    'Show toplevel window which\n' +
                    'will not get a busy cursor',
	        command = self.showExcludedWindow)
	w.pack(padx = 8, pady = 8)

	# Create the dialog.
	self.dialog = Pmw.Dialog(parent,
	    buttons = ('OK', 'Apply', 'Cancel', 'Help'),
	    defaultbutton = 'OK',
	    title = 'My dialog',
	    command = self.execute)
	self.dialog.withdraw()

	# Add some contents to the dialog.
	w = Tkinter.Label(self.dialog.interior(),
	    text = 'Pmw Dialog\n(put your widgets here)',
	    background = 'black',
	    foreground = 'white',
	    pady = 20)
	w.pack(expand = 1, fill = 'both', padx = 4, pady = 4)

	# Create the window excluded from showbusycursor.
	self.excluded = Pmw.MessageDialog(parent,
	    title = 'I still work',
	    message_text =
                'This window will not get\n' +
                'a busy cursor when modal dialogs\n' +
                'are activated.  In addition,\n' +
                'you can still interact with\n' +
                'this window when a "no grab"\n' +
                'modal dialog is displayed.')
	self.excluded.withdraw()
        Pmw.setbusycursorattributes(self.excluded.component('hull'),
            exclude = 1)

    def showAppModal(self):
        self.dialog.activate(geometry = 'centerscreenalways')

    def showGlobalModal(self):
        self.dialog.activate(globalMode = 1)

    def showDialogNoGrab(self):
        self.dialog.activate(globalMode = 'nograb')

    def showExcludedWindow(self):
        self.excluded.show()

    def execute(self, result):
	print 'You clicked on', result
	if result not in ('Apply', 'Help'):
	    self.dialog.deactivate(result)

######################################################################

# Create demo in root window for testing.
if __name__ == '__main__':
    root = Tkinter.Tk()
    Pmw.initialise(root, fontScheme = 'pmw1')
    root.title(title)

    exitButton = Tkinter.Button(root, text = 'Exit', command = root.destroy)
    exitButton.pack(side = 'bottom')
    widget = Demo(root)
    root.mainloop()



title = 'Pmw.ScrolledText demonstration'

# Import Pmw from this directory tree.
import sys
sys.path[:0] = ['../../..']

import os
import Tkinter
import Pmw

class Demo:
    def __init__(self, parent):
	# Create the ScrolledText.
	self.st = Pmw.ScrolledText(parent,
		borderframe = 1,
		labelpos = 'n',
		label_text='ScrolledText.py',
		usehullsize = 1,
		hull_width = 400,
		hull_height = 300,
		text_padx = 10,
		text_pady = 10,
		text_wrap='none'
	)

	# Create a group widget to contain the scrollmode options.
	w = Pmw.Group(parent, tag_text='Scroll mode')
	w.pack(side = 'bottom', padx = 5, pady = 5)

	hmode = Pmw.OptionMenu(w.interior(),
		labelpos = 'w',
		label_text = 'Horizontal:',
		items = ['none', 'static', 'dynamic'],
		command = self.sethscrollmode,
		menubutton_width = 8,
	)
	hmode.pack(side = 'left', padx = 5, pady = 5)
	hmode.invoke('dynamic')

	vmode = Pmw.OptionMenu(w.interior(),
		labelpos = 'w',
		label_text = 'Vertical:',
		items = ['none', 'static', 'dynamic'],
		command = self.setvscrollmode,
		menubutton_width = 8,
	)
	vmode.pack(side = 'left', padx = 5, pady = 5)
	vmode.invoke('dynamic')

	buttonBox = Pmw.ButtonBox(parent)
	buttonBox.pack(side = 'bottom')
	buttonBox.add('yview', text = 'Show\nyview', command = self.showYView)
	buttonBox.add('scroll', text = 'Page\ndown', command = self.pageDown)
	buttonBox.add('center', text = 'Center', command = self.centerPage)

	# Pack this last so that the buttons do not get shrunk when
	# the window is resized.
	self.st.pack(padx = 5, pady = 5, fill = 'both', expand = 1)

	# Read this file into the text widget.
	head, tail = os.path.split(sys.argv[0])
	self.st.importfile(os.path.join(head,'ScrolledText.py'))

	self.st.insert('end', '\nThis demonstrates how to\n' +
	    'add a window to a text widget:  ')
	counter = Pmw.Counter(self.st.component('text'),
	    entryfield_value = 9999)
	self.st.window_create('end', window = counter)

    def sethscrollmode(self, tag):
	self.st.configure(hscrollmode = tag)

    def setvscrollmode(self, tag):
	self.st.configure(vscrollmode = tag)

    def showYView(self):
        print self.st.yview()

    def pageDown(self):
        self.st.yview('scroll', 1, 'page')

    def centerPage(self):
        top, bottom = self.st.yview()
        size = bottom - top
        middle = 0.5 - size / 2
        self.st.yview('moveto', middle)

######################################################################

# Create demo in root window for testing.
if __name__ == '__main__':
    root = Tkinter.Tk()
    Pmw.initialise(root, fontScheme = 'pmw1')
    root.title(title)

    exitButton = Tkinter.Button(root, text = 'Exit', command = root.destroy)
    exitButton.pack(side = 'bottom')
    widget = Demo(root)
    root.mainloop()



# Based on iwidgets2.2.0/entryfield.itk code.

import re
import string
import types
import Tkinter
import Pmw

# Possible return values of validation functions.
OK = 1
ERROR = 0
PARTIAL = -1

class EntryField(Pmw.MegaWidget):
    _classBindingsDefinedFor = 0

    def __init__(self, parent = None, **kw):

	# Define the megawidget options.
	INITOPT = Pmw.INITOPT
	optiondefs = (
	    ('command',           None,        None),
	    ('errorbackground',   'pink',      None),
	    ('invalidcommand',    self.bell,   None),
	    ('labelmargin',       0,           INITOPT),
	    ('labelpos',          None,        INITOPT),
	    ('modifiedcommand',   None,        None),
	    ('validate',          None,        self._validate),
	    ('extravalidators',   {},          None),
	    ('value',             '',          INITOPT),
	)
	self.defineoptions(kw, optiondefs)

	# Initialise the base class (after defining the options).
	Pmw.MegaWidget.__init__(self, parent)

	# Create the components.
	interior = self.interior()
	self._entryFieldEntry = self.createcomponent('entry',
		(), None,
		Tkinter.Entry, (interior,))
	self._entryFieldEntry.grid(column=2, row=2, sticky='nsew')
	if self['value'] != '':
	    self.__setEntry(self['value'])
	interior.grid_columnconfigure(2, weight=1)
	interior.grid_rowconfigure(2, weight=1)

	self.createlabel(interior)

	# Initialise instance variables.

	self.normalBackground = None
        self._previousText = None

	# Initialise instance.

	_registerEntryField(self._entryFieldEntry, self)

        # Establish the special class bindings if not already done.
        # Also create bindings if the Tkinter default interpreter has
        # changed.  Use Tkinter._default_root to create class
        # bindings, so that a reference to root is created by
        # bind_class rather than a reference to self, which would
        # prevent object cleanup.
        if EntryField._classBindingsDefinedFor != Tkinter._default_root:
	    tagList = self._entryFieldEntry.bindtags()
            root  = Tkinter._default_root
	    	    
	    allSequences = {}
	    for tag in tagList:

                sequences = root.bind_class(tag)
                if type(sequences) is types.StringType:
                    # In old versions of Tkinter, bind_class returns a string
                    sequences = root.tk.splitlist(sequences)

		for sequence in sequences:
		    allSequences[sequence] = None
	    for sequence in allSequences.keys():
		root.bind_class('EntryFieldPre', sequence, _preProcess)
		root.bind_class('EntryFieldPost', sequence, _postProcess)

	    EntryField._classBindingsDefinedFor = root

	self._entryFieldEntry.bindtags(('EntryFieldPre',) +
		self._entryFieldEntry.bindtags() + ('EntryFieldPost',))
	self._entryFieldEntry.bind('<Return>', self._executeCommand)

	# Check keywords and initialise options.
	self.initialiseoptions()

    def destroy(self):
	_deregisterEntryField(self._entryFieldEntry)
        Pmw.MegaWidget.destroy(self)

    def _getValidatorFunc(self, validator, index):
	# Search the extra and standard validator lists for the
	# given 'validator'.  If 'validator' is an alias, then
	# continue the search using the alias.  Make sure that
	# self-referencial aliases do not cause infinite loops.

	extraValidators = self['extravalidators']
	traversedValidators = []

	while 1:
	    traversedValidators.append(validator)
	    if extraValidators.has_key(validator):
		validator = extraValidators[validator][index]
	    elif _standardValidators.has_key(validator):
		validator = _standardValidators[validator][index]
	    else:
		return validator
	    if validator in traversedValidators:
		return validator

    def _validate(self):
	dict = {
	    'validator' : None,
	    'min' : None,
	    'max' : None,
	    'minstrict' : 1,
	    'maxstrict' : 1,
	}
	opt = self['validate']
	if type(opt) is types.DictionaryType:
	    dict.update(opt)
	else:
	    dict['validator'] = opt

	# Look up validator maps and replace 'validator' field with
	# the corresponding function.
	validator = dict['validator']
	valFunction = self._getValidatorFunc(validator, 0)
	self._checkValidateFunction(valFunction, 'validate', validator)
	dict['validator'] = valFunction

	# Look up validator maps and replace 'stringtovalue' field
	# with the corresponding function.
	if dict.has_key('stringtovalue'):
	    stringtovalue = dict['stringtovalue'] 
	    strFunction = self._getValidatorFunc(stringtovalue, 1)
	    self._checkValidateFunction(
		    strFunction, 'stringtovalue', stringtovalue)
	else:
	    strFunction = self._getValidatorFunc(validator, 1)
	    if strFunction == validator:
		strFunction = len
	dict['stringtovalue'] = strFunction

	self._validationInfo = dict
	args = dict.copy()
	del args['validator']
	del args['min']
	del args['max']
	del args['minstrict']
	del args['maxstrict']
	del args['stringtovalue']
	self._validationArgs = args
        self._previousText = None

	if type(dict['min']) == types.StringType and strFunction is not None:
	    dict['min'] = apply(strFunction, (dict['min'],), args)
	if type(dict['max']) == types.StringType and strFunction is not None:
	    dict['max'] = apply(strFunction, (dict['max'],), args)

	self._checkValidity()

    def _checkValidateFunction(self, function, option, validator):
	# Raise an error if 'function' is not a function or None.

	if function is not None and not callable(function):
	    extraValidators = self['extravalidators']
	    extra = extraValidators.keys()
	    extra.sort()
	    extra = tuple(extra)
	    standard = _standardValidators.keys()
	    standard.sort()
	    standard = tuple(standard)
	    msg = 'bad %s value "%s":  must be a function or one of ' \
		'the standard validators %s or extra validators %s'
	    raise ValueError, msg % (option, validator, standard, extra)

    def _executeCommand(self, event = None):
	cmd = self['command']
	if callable(cmd):
            if event is None:
                # Return result of command for invoke() method.
                return cmd()
            else:
                cmd()
	    
    def _preProcess(self):

        self._previousText = self._entryFieldEntry.get()
        self._previousICursor = self._entryFieldEntry.index('insert')
        self._previousXview = self._entryFieldEntry.index('@0')
	if self._entryFieldEntry.selection_present():
	    self._previousSel= (self._entryFieldEntry.index('sel.first'),
		self._entryFieldEntry.index('sel.last'))
	else:
	    self._previousSel = None

    def _postProcess(self):

	# No need to check if text has not changed.
	previousText = self._previousText
	if previousText == self._entryFieldEntry.get():
	    return self.valid()

	valid = self._checkValidity()
        if self.hulldestroyed():
            # The invalidcommand called by _checkValidity() destroyed us.
            return valid

	cmd = self['modifiedcommand']
	if callable(cmd) and previousText != self._entryFieldEntry.get():
	    cmd()
	return valid
	    
    def checkentry(self):
	# If there is a variable specified by the entry_textvariable
	# option, checkentry() should be called after the set() method
	# of the variable is called.

	self._previousText = None
	return self._postProcess()

    def _getValidity(self):
	text = self._entryFieldEntry.get()
	dict = self._validationInfo
	args = self._validationArgs

	if dict['validator'] is not None:
	    status = apply(dict['validator'], (text,), args)
	    if status != OK:
		return status

	# Check for out of (min, max) range.
	if dict['stringtovalue'] is not None:
	    min = dict['min']
	    max = dict['max']
	    if min is None and max is None:
		return OK
	    val = apply(dict['stringtovalue'], (text,), args)
	    if min is not None and val < min:
		if dict['minstrict']:
		    return ERROR
		else:
		    return PARTIAL
	    if max is not None and val > max:
		if dict['maxstrict']:
		    return ERROR
		else:
		    return PARTIAL
	return OK

    def _checkValidity(self):
	valid = self._getValidity()
	oldValidity = valid

	if valid == ERROR:
	    # The entry is invalid.
	    cmd = self['invalidcommand']
	    if callable(cmd):
		cmd()
            if self.hulldestroyed():
                # The invalidcommand destroyed us.
                return oldValidity

	    # Restore the entry to its previous value.
	    if self._previousText is not None:
		self.__setEntry(self._previousText)
		self._entryFieldEntry.icursor(self._previousICursor)
		self._entryFieldEntry.xview(self._previousXview)
		if self._previousSel is not None:
		    self._entryFieldEntry.selection_range(self._previousSel[0],
			self._previousSel[1])

		# Check if the saved text is valid as well.
		valid = self._getValidity()

	self._valid = valid

        if self.hulldestroyed():
            # The validator or stringtovalue commands called by
            # _checkValidity() destroyed us.
            return oldValidity

	if valid == OK:
	    if self.normalBackground is not None:
		self._entryFieldEntry.configure(
			background = self.normalBackground)
		self.normalBackground = None
	else:
	    if self.normalBackground is None:
		self.normalBackground = self._entryFieldEntry.cget('background')
		self._entryFieldEntry.configure(
			background = self['errorbackground'])

        return oldValidity

    def invoke(self):
	return self._executeCommand()

    def valid(self):
        return self._valid == OK

    def clear(self):
        self.setentry('')

    def __setEntry(self, text):
	disabled = (self._entryFieldEntry.cget('state') == 'disabled')
	if disabled:
	    self._entryFieldEntry.configure(state='normal')
	self._entryFieldEntry.delete(0, 'end')
	self._entryFieldEntry.insert(0, text)
	if disabled:
	    self._entryFieldEntry.configure(state='disabled')

    def setentry(self, text):
	self._preProcess()
        self.__setEntry(text)
	return self._postProcess()

    def getvalue(self):
        return self._entryFieldEntry.get()

    def setvalue(self, text):
        return self.setentry(text)

Pmw.forwardmethods(EntryField, Tkinter.Entry, '_entryFieldEntry')

# ======================================================================


# Entry field validation functions

_numericregex = re.compile('^[0-9]*$')
_alphabeticregex = re.compile('^[a-z]*$', re.IGNORECASE)
_alphanumericregex = re.compile('^[0-9a-z]*$', re.IGNORECASE)

def numericvalidator(text):
    if text == '':
        return PARTIAL
    else:
	if _numericregex.match(text) is None:
	    return ERROR
	else:
	    return OK
    
def integervalidator(text):
    if text in ('', '-', '+'):
        return PARTIAL
    try:
	string.atol(text)
	return OK
    except ValueError:
	return ERROR
    
def alphabeticvalidator(text):
    if _alphabeticregex.match(text) is None:
	return ERROR
    else:
	return OK
    
def alphanumericvalidator(text):
    if _alphanumericregex.match(text) is None:
	return ERROR
    else:
	return OK
    
def hexadecimalvalidator(text):
    if text in ('', '0x', '0X', '+', '+0x', '+0X', '-', '-0x', '-0X'):
        return PARTIAL
    try:
	string.atol(text, 16)
	return OK
    except ValueError:
	return ERROR
    
def realvalidator(text, separator = '.'):
    if separator != '.':
	if string.find(text, '.') >= 0:
	    return ERROR
	index = string.find(text, separator)
	if index >= 0:
	    text = text[:index] + '.' + text[index + 1:]
    try:
	string.atof(text)
	return OK
    except ValueError:
	# Check if the string could be made valid by appending a digit
	# eg ('-', '+', '.', '-.', '+.', '1.23e', '1E-').
	if len(text) == 0:
	    return PARTIAL
	if text[-1] in string.digits:
	    return ERROR
	try:
	    string.atof(text + '0')
	    return PARTIAL
	except ValueError:
	    return ERROR
    
def timevalidator(text, separator = ':'):
    try:
	Pmw.timestringtoseconds(text, separator)
	return OK
    except ValueError:
	if len(text) > 0 and text[0] in ('+', '-'):
	    text = text[1:]
	if re.search('[^0-9' + separator + ']', text) is not None:
	    return ERROR
	return PARTIAL

def datevalidator(text, format = 'ymd', separator = '/'):
    try:
	Pmw.datestringtojdn(text, format, separator)
	return OK
    except ValueError:
	if re.search('[^0-9' + separator + ']', text) is not None:
	    return ERROR
	return PARTIAL

_standardValidators = {
    'numeric'      : (numericvalidator,      string.atol),
    'integer'      : (integervalidator,      string.atol),
    'hexadecimal'  : (hexadecimalvalidator,  lambda s: string.atol(s, 16)),
    'real'         : (realvalidator,         Pmw.stringtoreal),
    'alphabetic'   : (alphabeticvalidator,   len),
    'alphanumeric' : (alphanumericvalidator, len),
    'time'         : (timevalidator,         Pmw.timestringtoseconds),
    'date'         : (datevalidator,         Pmw.datestringtojdn),
}

_entryCache = {}

def _registerEntryField(entry, entryField):
    # Register an EntryField widget for an Entry widget

    _entryCache[entry] = entryField

def _deregisterEntryField(entry):
    # Deregister an Entry widget
    del _entryCache[entry]

def _preProcess(event):
    # Forward preprocess events for an Entry to it's EntryField

    _entryCache[event.widget]._preProcess()

def _postProcess(event):
    # Forward postprocess events for an Entry to it's EntryField

    # The function specified by the 'command' option may have destroyed
    # the megawidget in a binding earlier in bindtags, so need to check.
    if _entryCache.has_key(event.widget):
        _entryCache[event.widget]._postProcess()



"""
I needed a simple gauge, so I've made on with Pmw.
It might be useful for others to use as a base to develop more comples
gauges with.
 
Is it worth cleaning up and submitting?
 
cheers and thanks
 
chris
 
Dr. Chris Wright
Intensive Care Unit
Monash Medical Centre
Clayton. VIC Australia
"""

import sys
import Tkinter
import Pmw
import time


if sys.platform == 'win32':
    # MS-Windows specific fonts
    label_font = "-family Ariel  -size 12"
    value_font = "-family Ariel  -size 12"
    small_font = "-family {MS Sans Serif} -size 9 -weight bold"
    header_font = "-family {MS Sans Serif} -weight bold"
else:
    # X-Windows specific fonts
    label_font = "-*-helvetica-*-r-*-*-*-160-*-*-*-*-*-*"
    value_font = "-*-courier-*-r-*-*-*-160-*-*-*-*-*-*"
    small_font = "-*-helvetica-*-r-*-*-*-130-*-*-*-*-*-*"
    header_font = "-*-helvetica-bold-r-*-*-*-150-*-*-*-*-*-*"

class VerticalGauge(Pmw.MegaWidget):
    """Vertical gauge with actual and desired settings"""

    def __init__(self, parent = None, **kw):
	optiondefs = (
	    ('min', 0, None),
	    ('max', 100, None),
	    ('majortickinterval', 10, None),
	    ('minortickinterval', 5, None),
	    ('units', '', None),
	    ('bg', 'grey', self._backgroundSet),
	    ('actualvalue', 50, self._actualSet),
	    ('desiredvalue', 50, self._desiredSet),
	    ('actualcolour', 'yellow1', None),
	    ('desiredcolour', 'turquoise1', None),
	    ('label', 'Label', None),
	    )
	self.defineoptions(kw, optiondefs)
	Pmw.MegaWidget.__init__(self, parent)

	interior = self.interior()
	interior.grid_rowconfigure(1, weight = 1)
	for r in range(3):
	    interior.grid_columnconfigure(r, weight = 1)

	self.actuallabel = self.createcomponent('actualLabel',
						(), None,
						Tkinter.Label, (interior,),
						text = '',
						width = 3,
						relief = 'sunken',
						bd = 1,
						fg = self['actualcolour'],
						font = value_font)
	self.actuallabel.grid(sticky = "nswe", row = 0, column = 0)

	self.label = self.createcomponent('label',
					  (), None,
					  Tkinter.Label, (interior,),
					  text = self['label'],
					  relief = 'raised',
					  font = label_font,
					  fg = 'navy',
					  bd = 2)
	self.label.grid(sticky = "nsew", row = 0, column = 1)

	self.desiredlabel = self.createcomponent('desiredLabel',
						 (), None,
						 Tkinter.Label, (interior,),
						 text = '',
						 width = 3,
						 relief = 'sunken',
						 bd = 1,
						 fg = self['desiredcolour'],
						 font = value_font)
	self.desiredlabel.grid(sticky = "nswe", row = 0, column = 2)

	self.canvas = self.createcomponent('canvas',
					   (), None,
					   Tkinter.Canvas, (interior,),
					   width = 100,
					   height = 300,
					   bg = 'grey')
			
	self.canvas.grid(sticky = "nsew", columnspan = 3, pady = 1)
	self.canvas.bind("<Configure>", self._createGaugeAxes)
	
	self._createGaugeAxes()
	
	self.initialiseoptions()
    
    def _createGaugeAxes(self, event = None):
	min = self['min']
	max = self['max']
	units = self['units']
	majortickinterval = self['majortickinterval']

	gauge_range = max - min

	c = self.canvas
	c.delete("all")
	if event:
	    h, w = event.height, event.width
	else:
	    h = int(c.configure("height")[4])
	    w = int(c.configure("width")[4])

	self.lower = h - 15
	self.upper = 15
	self.middle = w / 2
	c.create_line(self.middle, self.lower, self.middle, self.upper)
	
	majortickcount = int((max - min) / majortickinterval) 
	self.axislength = self.lower - self.upper
	self.majortickdistance = float(self.axislength) / majortickcount
	self.majortickwidth = w / 5
	labeloffset = (w / 4) + 10

	for i in range(majortickcount + 1):
	    v = min + i * majortickinterval
	    d = self.lower - i * self.majortickdistance
	    c.create_line(self.middle, d, self.middle + self.majortickwidth, d)
	    c.create_text(self.middle + labeloffset, d, font = small_font, text = str(v))
	
	self._desiredSet(event)
	self._actualSet(event)

    def _backgroundSet(self):
	self.canvas.configure(bg = self['bg'])
		
    def _desiredSet(self, event = None):
	c = self.canvas
	desired = self['desiredvalue']
	desiredcolour = self['desiredcolour']

	min = self['min']
	max = self['max']
	
	if desired > max: desired = max
	if desired < min: desired = min
	gauge_range = max - min
	
	c = self.canvas
	if event:
	    h, w = event.height, event.width
	else:
	    h = int(c.configure("height")[4])
	    w = int(c.configure("width")[4])


	desired_y = self.lower - (float(desired - min) / gauge_range) * self.axislength

	try:
	    c.delete('desiredBar')
	except:
	    pass

	c.create_line(self.middle - self.majortickwidth, desired_y, 
		      self.middle + self.majortickwidth, desired_y,
		      fill = desiredcolour, stipple = 'gray50',
		      width = 10, tag = 'desiredBar')
	self.desiredlabel.configure(text = desired)

    def setActual(self, value):
	self.configure(actualvalue = value)

    def getActual(self):
	return self.cget('actualvalue')

    def _actualSet(self, event = None):
	c = self.canvas
	actual = self['actualvalue']
	actualcolour = self['actualcolour']

	min = self['min']
	max = self['max']
	
	if actual > max: actual = max
	if actual < min: actual = min
	gauge_range = max - min
	
	c = self.canvas
	if event:
	    h, w = event.height, event.width
	else:
	    h = int(c.configure("height")[4])
	    w = int(c.configure("width")[4])

	actual_y = self.lower - (float(actual - min) / gauge_range) * self.axislength

	try:
	    c.delete('actualPointer')
	except:
	    pass
	
	triangle = ((self.middle, actual_y), 
		    (self.middle - 1.4 * self.majortickwidth, actual_y - self.majortickwidth / 2),
		    (self.middle - 1.4 * self.majortickwidth, actual_y + self.majortickwidth / 2))

	c.create_polygon(triangle, fill = actualcolour, tag = 'actualPointer')
	self.actuallabel.configure(text = actual)


Pmw.forwardmethods(VerticalGauge, Tkinter.Canvas, 'canvas')

if __name__ == '__main__':


    # Initialise Tkinter and Pmw.
    root = Pmw.initialise()
    root.title('Pmw VerticalGauge demonstration')


    def increase():
	av = g1.getActual()
	g1.setActual(av + 1)

    def decrease():
	av = g1.getActual()
	g1.setActual(av - 1)

    g1 = VerticalGauge(min = 0,
		       max = 30, 
		       actualvalue = 15,
		       desiredvalue = 22,
		       majortickinterval = 2,
		       label = "Pms")
    g1.grid(sticky = "nsew")
    root.grid_rowconfigure(0, weight = 1)
    root.grid_columnconfigure(0, weight = 1)
    b1 = Tkinter.Button(text = "Increase", command = increase)
    b1.grid()
    b2 = Tkinter.Button(text = "Decrease", command = decrease)
    b2.grid()

    # Let's go.
    root.mainloop()



title = 'Pmw.RadioSelect demonstration'

# Import Pmw from this directory tree.
import sys
sys.path[:0] = ['../../..']

import Tkinter
import Pmw

class Demo:
    def __init__(self, parent):
	# Create and pack a horizontal RadioSelect widget.
	horiz = Pmw.RadioSelect(parent,
		labelpos = 'w',
		command = self.callback,
		label_text = 'Horizontal',
		frame_borderwidth = 2,
		frame_relief = 'ridge'
	)
	horiz.pack(fill = 'x', padx = 10, pady = 10)

	# Add some buttons to the horizontal RadioSelect.
	for text in ('Fruit', 'Vegetables', 'Cereals', 'Legumes'):
	    horiz.add(text)
	horiz.invoke('Cereals')

	# Create and pack a multiple selection RadioSelect widget.
	self.multiple = Pmw.RadioSelect(parent,
		labelpos = 'w',
		command = self.multcallback,
		label_text = 'Multiple\nselection',
		frame_borderwidth = 2,
		frame_relief = 'ridge',
		selectmode = 'multiple',
	)
	self.multiple.pack(fill = 'x', padx = 10)

	# Add some buttons to the multiple selection RadioSelect.
	for text in ('Apricots', 'Eggplant', 'Rice', 'Lentils'):
	    self.multiple.add(text)
	self.multiple.invoke('Rice')

	# Create and pack a vertical RadioSelect widget, with checkbuttons.
	self.checkbuttons = Pmw.RadioSelect(parent,
		buttontype = 'checkbutton',
		orient = 'vertical',
		labelpos = 'w',
		command = self.checkbuttoncallback,
		label_text = 'Vertical,\nusing\ncheckbuttons',
		hull_borderwidth = 2,
		hull_relief = 'ridge',
	)
	self.checkbuttons.pack(side = 'left', expand = 1, padx = 10, pady = 10)

	# Add some buttons to the checkbutton RadioSelect.
	for text in ('Male', 'Female'):
	    self.checkbuttons.add(text)
	self.checkbuttons.invoke('Male')
	self.checkbuttons.invoke('Female')

	# Create and pack a RadioSelect widget, with radiobuttons.
	radiobuttons = Pmw.RadioSelect(parent,
		buttontype = 'radiobutton',
		orient = 'vertical',
		labelpos = 'w',
		command = self.callback,
		label_text = 'Vertical,\nusing\nradiobuttons',
		hull_borderwidth = 2,
		hull_relief = 'ridge',
	)
	radiobuttons.pack(side = 'left', expand = 1, padx = 10, pady = 10)

	# Add some buttons to the radiobutton RadioSelect.
	for text in ('Male', 'Female', 'Both', 'Neither'):
	    radiobuttons.add(text)
	radiobuttons.invoke('Both')

    def callback(self, tag):
	# This is called whenever the user clicks on a button
	# in a single select RadioSelect widget.
	print 'Button', tag, 'was pressed.'

    def multcallback(self, tag, state):
	# This is called whenever the user clicks on a button
	# in the multiple select RadioSelect widget.
        if state:
           action = 'pressed.'
        else:
           action = 'released.'

        print 'Button', tag, 'was', action, \
		'Selection:', self.multiple.getcurselection()
           
    def checkbuttoncallback(self, tag, state):
	# This is called whenever the user clicks on a button
	# in the checkbutton RadioSelect widget.
        if state:
           action = 'pressed.'
        else:
           action = 'released.'

        print 'Button', tag, 'was', action, \
		'Selection:', self.checkbuttons.getcurselection()
           
######################################################################

# Create demo in root window for testing.
if __name__ == '__main__':
    root = Tkinter.Tk()
    Pmw.initialise(root, fontScheme = 'pmw1')
    root.title(title)

    exitButton = Tkinter.Button(root, text = 'Exit', command = root.destroy)
    exitButton.pack(side = 'bottom')
    widget = Demo(root)
    root.mainloop()



# Based on iwidgets2.2.0/dialog.itk and iwidgets2.2.0/dialogshell.itk code.

# Convention:
#   Each dialog window should have one of these as the rightmost button:
#     Close         Close a window which only displays information.
#     Cancel        Close a window which may be used to change the state of
#                   the application.

import sys
import types
import Tkinter
import Pmw

# A Toplevel with a ButtonBox and child site.

class Dialog(Pmw.MegaToplevel):
    def __init__(self, parent = None, **kw):

	# Define the megawidget options.
	INITOPT = Pmw.INITOPT
	optiondefs = (
	    ('buttonbox_hull_borderwidth',   1,         None),
	    ('buttonbox_hull_relief',        'raised',  None),
	    ('buttonboxpos',                 's',       INITOPT),
	    ('buttons',                      ('OK',),   self._buttons),
	    ('command',                      None,      None),
	    ('dialogchildsite_borderwidth',  1,         None),
	    ('dialogchildsite_relief',       'raised',  None),
	    ('defaultbutton',                None,      self._defaultButton),
            ('master',                       'parent',  None),
	    ('separatorwidth',               0,         INITOPT),
	)
	self.defineoptions(kw, optiondefs)

	# Initialise the base class (after defining the options).
	Pmw.MegaToplevel.__init__(self, parent)

	# Create the components.

	oldInterior = Pmw.MegaToplevel.interior(self)

	# Set up pack options according to the position of the button box.
        pos = self['buttonboxpos']
	if pos not in 'nsew':
	    raise ValueError, \
	        'bad buttonboxpos option "%s":  should be n, s, e, or w' \
		    % pos

	if pos in 'ns':
	    orient = 'horizontal'
	    fill = 'x'
	    if pos == 'n':
	        side = 'top'
	    else:
	        side = 'bottom'
	else:
	    orient = 'vertical'
	    fill = 'y'
	    if pos == 'w':
	        side = 'left'
	    else:
	        side = 'right'

	# Create the button box.
	self._buttonBox = self.createcomponent('buttonbox',
		(), None,
		Pmw.ButtonBox, (oldInterior,), orient = orient)
	self._buttonBox.pack(side = side, fill = fill)

	# Create the separating line.
	width = self['separatorwidth']
	if width > 0:
	    self._separator = self.createcomponent('separator',
		    (), None,
		    Tkinter.Frame, (oldInterior,), relief = 'sunken',
		    height = width, width = width, borderwidth = width / 2)
	    self._separator.pack(side = side, fill = fill)
	
	# Create the child site.
	self.__dialogChildSite = self.createcomponent('dialogchildsite',
		(), None,
		Tkinter.Frame, (oldInterior,))
	self.__dialogChildSite.pack(side=side, fill='both', expand=1)

	self.oldButtons = ()
	self.oldDefault = None

	self.bind('<Return>', self._invokeDefault)
        self.userdeletefunc(self._doCommand)
        self.usermodaldeletefunc(self._doCommand)
	
	# Check keywords and initialise options.
	self.initialiseoptions()

    def interior(self):
	return self.__dialogChildSite

    def invoke(self, index = Pmw.DEFAULT):
	return self._buttonBox.invoke(index)

    def _invokeDefault(self, event):
	try:
	    self._buttonBox.index(Pmw.DEFAULT)
	except ValueError:
	    return
	self._buttonBox.invoke()

    def _doCommand(self, name = None):
        if name is not None and self.active() and \
                Pmw.grabstacktopwindow() != self.component('hull'):
            # This is a modal dialog but is not on the top of the grab
            # stack (ie:  should not have the grab), so ignore this
            # event.  This seems to be a bug in Tk and may occur in
            # nested modal dialogs.
            #
            # An example is the PromptDialog demonstration.  To
            # trigger the problem, start the demo, then move the mouse
            # to the main window, hit <TAB> and then <TAB> again.  The
            # highlight border of the "Show prompt dialog" button
            # should now be displayed.  Now hit <SPACE>, <RETURN>,
            # <RETURN> rapidly several times.  Eventually, hitting the
            # return key invokes the password dialog "OK" button even
            # though the confirm dialog is active (and therefore
            # should have the keyboard focus).  Observed under Solaris
            # 2.5.1, python 1.5.2 and Tk8.0.

            # TODO:  Give focus to the window on top of the grabstack.
            return

	command = self['command']
	if callable(command):
	    return command(name)
	else:
	    if self.active():
	        self.deactivate(name)
	    else:
	        self.withdraw()

    def _buttons(self):
	buttons = self['buttons']
	if type(buttons) != types.TupleType and type(buttons) != types.ListType:
	    raise ValueError, \
	        'bad buttons option "%s": should be a tuple' % str(buttons)
	if self.oldButtons == buttons:
	  return

	self.oldButtons = buttons

	for index in range(self._buttonBox.numbuttons()):
	    self._buttonBox.delete(0)
	for name in buttons:
	    self._buttonBox.add(name,
		command=lambda self=self, name=name: self._doCommand(name))

	if len(buttons) > 0:
	    defaultbutton = self['defaultbutton']
	    if defaultbutton is None:
		self._buttonBox.setdefault(None)
	    else:
		try:
		    self._buttonBox.index(defaultbutton)
		except ValueError:
		    pass
		else:
		    self._buttonBox.setdefault(defaultbutton)
	self._buttonBox.alignbuttons()

    def _defaultButton(self):
	defaultbutton = self['defaultbutton']
	if self.oldDefault == defaultbutton:
	  return

	self.oldDefault = defaultbutton

	if len(self['buttons']) > 0:
	    if defaultbutton is None:
		self._buttonBox.setdefault(None)
	    else:
		try:
		    self._buttonBox.index(defaultbutton)
		except ValueError:
		    pass
		else:
		    self._buttonBox.setdefault(defaultbutton)



import Tkinter
import Pmw

class LabeledWidget(Pmw.MegaWidget):
    def __init__(self, parent = None, **kw):

	# Define the megawidget options.
	INITOPT = Pmw.INITOPT
	optiondefs = (
	    ('labelmargin',            0,      INITOPT),
	    ('labelpos',               None,   INITOPT),
	)
	self.defineoptions(kw, optiondefs)

	# Initialise the base class (after defining the options).
	Pmw.MegaWidget.__init__(self, parent)

	# Create the components.
	interior = Pmw.MegaWidget.interior(self)
	self._labelChildSite = self.createcomponent('labelchildsite',
		(), None,
		Tkinter.Frame, (interior,))
	self._labelChildSite.grid(column=2, row=2, sticky='nsew')
	interior.grid_columnconfigure(2, weight=1)
	interior.grid_rowconfigure(2, weight=1)

	self.createlabel(interior)

	# Check keywords and initialise options.
	self.initialiseoptions()

    def interior(self):
	return self._labelChildSite



import string
import Tkinter
import Pmw

def aligngrouptags(groups):
    # Adjust the y position of the tags in /groups/ so that they all
    # have the height of the highest tag.

    maxTagHeight = 0
    for group in groups:
	if group._tag is None:
	    height = (string.atoi(group._ring.cget('borderwidth')) +
		    string.atoi(group._ring.cget('highlightthickness')))
	else:
	    height = group._tag.winfo_reqheight()
	if maxTagHeight < height:
	    maxTagHeight = height

    for group in groups:
	ringBorder = (string.atoi(group._ring.cget('borderwidth')) +
		string.atoi(group._ring.cget('highlightthickness')))
	topBorder = maxTagHeight / 2 - ringBorder / 2
	group._hull.grid_rowconfigure(0, minsize = topBorder)
	group._ring.grid_rowconfigure(0,
		minsize = maxTagHeight - topBorder - ringBorder)
	if group._tag is not None:
	    group._tag.place(y = maxTagHeight / 2)

class Group( Pmw.MegaWidget ):
    def __init__(self, parent = None, **kw):

        # Define the megawidget options.
	INITOPT = Pmw.INITOPT
        optiondefs = (
	    ('ring_borderwidth', 2,         None),
	    ('ring_relief',      'groove',  None),
	    ('tagindent',        10,         INITOPT),
	    )
        self.defineoptions(kw, optiondefs)

        # Initialise the base class (after defining the options).
        Pmw.MegaWidget.__init__(self, parent)

        # Create the components.
        interior = Pmw.MegaWidget.interior(self)

	self._ring = self.createcomponent(
	    'ring', 
	    (), None,
	    Tkinter.Frame, (interior,), 
	    )

	self._groupChildSite = self.createcomponent(
	    'groupchildsite',
	    (), None,
	    Tkinter.Frame, (self._ring,)
	    )

        self._tag = self.createcomponent(
	    'tag',
	    (), None,
	    Tkinter.Label, (interior,),
	    )

	ringBorder = (string.atoi(self._ring.cget('borderwidth')) +
		string.atoi(self._ring.cget('highlightthickness')))
	if self._tag is None:
	    tagHeight = ringBorder
	else:
	    tagHeight = self._tag.winfo_reqheight()
	    self._tag.place(
		    x = ringBorder + self['tagindent'],
		    y = tagHeight / 2,
		    anchor = 'w')

	topBorder = tagHeight / 2 - ringBorder / 2
	self._ring.grid(column = 0, row = 1, sticky = 'nsew')
	interior.grid_columnconfigure(0, weight = 1)
	interior.grid_rowconfigure(1, weight = 1)
	interior.grid_rowconfigure(0, minsize = topBorder)

	self._groupChildSite.grid(column = 0, row = 1, sticky = 'nsew')
	self._ring.grid_columnconfigure(0, weight = 1)
	self._ring.grid_rowconfigure(1, weight = 1)
	self._ring.grid_rowconfigure(0,
		minsize = tagHeight - topBorder - ringBorder)

        # Check keywords and initialise options.
        self.initialiseoptions()

    def interior(self):
        return self._groupChildSite



# Python interface to some of the commands of the 2.4 version of the
# BLT extension to tcl.

import string
import types
import Tkinter

# Supported commands:
_busyCommand = '::blt::busy'
_vectorCommand = '::blt::vector'
_graphCommand = '::blt::graph'
_testCommand = '::blt::*'
_chartCommand = '::blt::stripchart'
_tabsetCommand = '::blt::tabset'

_haveBlt = None
_haveBltBusy = None

def _checkForBlt(window):
    global _haveBlt
    global _haveBltBusy

    # Blt may be a package which has not yet been loaded. Try to load it.
    try:
	window.tk.call('package', 'require', 'BLT')
    except Tkinter.TclError:
	# Another way to try to dynamically load blt:
	try:
	    window.tk.call('load', '', 'Blt')
	except Tkinter.TclError:
	    pass

    _haveBlt= (window.tk.call('info', 'commands', _testCommand) != '')
    _haveBltBusy = (window.tk.call('info', 'commands', _busyCommand) != '')

def haveblt(window):
    if _haveBlt is None:
	_checkForBlt(window)
    return _haveBlt

def havebltbusy(window):
    if _haveBlt is None:
	_checkForBlt(window)
    return _haveBltBusy

def _loadBlt(window):
    if _haveBlt is None:
	if window is None:
	    window = Tkinter._default_root
	    if window is None:
	    	window = Tkinter.Tk()
	_checkForBlt(window)

def busy_hold(window, cursor = None):
    _loadBlt(window)
    if cursor is None:
        window.tk.call(_busyCommand, 'hold', window._w)
    else:
        window.tk.call(_busyCommand, 'hold', window._w, '-cursor', cursor)

def busy_release(window):
    _loadBlt(window)
    window.tk.call(_busyCommand, 'release', window._w)

def busy_forget(window):
    _loadBlt(window)
    window.tk.call(_busyCommand, 'forget', window._w)

#=============================================================================
# Interface to the blt vector command which makes it look like the
# builtin python list type.
# The -variable, -command, -watchunset creation options are not supported.
# The dup, merge, notify, offset, populate, seq and variable methods
# and the +, -, * and / operations are not supported.

# Blt vector functions:
def vector_expr(expression):
    tk = Tkinter._default_root.tk
    strList = tk.splitlist(tk.call(_vectorCommand, 'expr', expression))
    return tuple(map(string.atof, strList))

def vector_names(pattern = None):
    tk = Tkinter._default_root.tk
    return tk.splitlist(tk.call(_vectorCommand, 'names', pattern))

class Vector:
    _varnum = 0
    def __init__(self, size=None, master=None):
        # <size> can be either an integer size, or a string "first:last".
	_loadBlt(master)
	if master:
	    self._master = master
	else:
	    self._master = Tkinter._default_root
	self.tk = self._master.tk
	self._name = 'PY_VEC' + str(Vector._varnum)
	Vector._varnum = Vector._varnum + 1
	if size is None:
	    self.tk.call(_vectorCommand, 'create', self._name)
	else:
	  self.tk.call(_vectorCommand, 'create', '%s(%s)' % (self._name, size))
    def __del__(self):
	self.tk.call(_vectorCommand, 'destroy', self._name)
    def __str__(self):
	return self._name

    def __repr__(self):
	return '[' + string.join(map(str, self), ', ') + ']'
    def __cmp__(self, list):
	return cmp(self[:], list)

    def __len__(self): 
	return self.tk.getint(self.tk.call(self._name, 'length'))
    def __getitem__(self, key): 
	oldkey = key
	if key < 0:
	    key = key + len(self)
	try:
	    return self.tk.getdouble(self.tk.globalgetvar(self._name, str(key)))
        except Tkinter.TclError:
	    raise IndexError, oldkey
    def __setitem__(self, key, value): 
	if key < 0:
	    key = key + len(self)
	return self.tk.globalsetvar(self._name, str(key), float(value))

    def __delitem__(self, key):
	if key < 0:
	    key = key + len(self)
	return self.tk.globalunsetvar(self._name, str(key))

    def __getslice__(self, start, end):
	length = len(self)
	if start < 0:
	    start = 0
	if end > length:
	    end = length
	if start >= end:
	    return []
	end = end - 1  # Blt vector slices include end point.
	text = self.tk.globalgetvar(self._name, str(start) + ':' + str(end))
	return map(self.tk.getdouble, self.tk.splitlist(text))

    def __setslice__(self, start, end, list):
	if start > end:
	    end = start
	self.set(self[:start] + list + self[end:])

    def __delslice__(self, start, end):
	if start < end:
	    self.set(self[:start] + self[end:])

    def __add__(self, list):
	return self[:] + list
    def __radd__(self, list):
	return list + self[:]
    def __mul__(self, n):
	return self[:] * n
    __rmul__ = __mul__

    # Python builtin list methods:
    def append(self, *args):
	self.tk.call(self._name, 'append', args)
    def count(self, obj):
	return self[:].count(obj)
    def index(self, value):
	return self[:].index(value)
    def insert(self, index, value):
	self[index:index] = [value]
    def remove(self, value):
	del self[self.index(value)]
    def reverse(self):
	s = self[:]
	s.reverse()
	self.set(s)
    def sort(self, *args):
	s = self[:]
	s.sort()
	self.set(s)

    # Blt vector instance methods:
    # append - same as list method above
    def clear(self):
	self.tk.call(self._name, 'clear')
    def delete(self, *args):
	self.tk.call((self._name, 'delete') + args)
    def expr(self, expression):
	self.tk.call(self._name, 'expr', expression)
    def length(self, newSize=None): 
	return self.tk.getint(self.tk.call(self._name, 'length', newSize))
    def range(self, first, last=None):
	# Note that, unlike self[first:last], this includes the last
	# item in the returned range.
	text = self.tk.call(self._name, 'range', first, last)
	return map(self.tk.getdouble, self.tk.splitlist(text))
    def search(self, start, end=None):
	return self._master._getints(self.tk.call(
		self._name, 'search', start, end))
    def set(self, list):
	if type(list) != types.TupleType:
	    list = tuple(list)
	self.tk.call(self._name, 'set', list)

    # The blt vector sort method has different semantics to the python
    # list sort method.  Call these blt_sort:
    def blt_sort(self, *args):
	self.tk.call((self._name, 'sort') + args)
    def blt_sort_reverse(self, *args):
	self.tk.call((self._name, 'sort', '-reverse') + args)

    # Special blt vector indexes:
    def min(self):
	return self.tk.getdouble(self.tk.globalgetvar(self._name, 'min'))
    def max(self):
	return self.tk.getdouble(self.tk.globalgetvar(self._name, 'max'))

    # Method borrowed from Tkinter.Var class:
    def get(self):
	return self[:]

#=============================================================================

# This is a general purpose configure routine which can handle the
# configuration of widgets, items within widgets, etc.  Supports the
# forms configure() and configure('font') for querying and
# configure(font = 'fixed', text = 'hello') for setting.

def _doConfigure(widget, subcommand, option, kw):

    if not option and not kw:
        # Return a description of all options.
        ret = {}
        options = widget.tk.splitlist(widget.tk.call(subcommand))
        for optionString in options:
            optionInfo = widget.tk.splitlist(optionString)
            option = optionInfo[0][1:]
            ret[option] = (option,) + optionInfo[1:]
        return ret

    if option:
        # Return a description of the option given by <option>.
        if kw:
            # Having keywords implies setting configuration options.
            # Can't set and get in one command!
            raise ValueError, 'cannot have option argument with keywords'
        option = '-' + option
        optionInfo = widget.tk.splitlist(widget.tk.call(subcommand + (option,)))
        return (optionInfo[0][1:],) + optionInfo[1:]

    # Otherwise, set the given configuration options.
    widget.tk.call(subcommand + widget._options(kw))

#=============================================================================

class Graph(Tkinter.Widget):
    # Wrapper for the blt graph widget, version 2.4.

    def __init__(self, master=None, cnf={}, **kw):
	_loadBlt(master)
	Tkinter.Widget.__init__(self, master, _graphCommand, cnf, kw)

    def bar_create(self, name, **kw):
	self.tk.call((self._w, 'bar', 'create', name) + self._options(kw))

    def line_create(self, name, **kw):
	self.tk.call((self._w, 'line', 'create', name) + self._options(kw))

    def extents(self, item):
	return self.tk.getint(self.tk.call(self._w, 'extents', item))

    def invtransform(self, winX, winY):
	return self._getdoubles(
		self.tk.call(self._w, 'invtransform', winX, winY))

    def inside(self, x, y):
	return self.tk.getint(self.tk.call(self._w, 'inside', x, y))

    def snap(self, photoName):
	self.tk.call(self._w, 'snap', photoName)

    def transform(self, x, y):
	return self._getdoubles(self.tk.call(self._w, 'transform', x, y))

    def axis_cget(self, axisName, key):
	return self.tk.call(self._w, 'axis', 'cget', axisName, '-' + key)
    def axis_configure(self, axes, option=None, **kw):
        # <axes> may be a list of axisNames.
	if type(axes) == types.StringType:
            axes = [axes]
	subcommand = (self._w, 'axis', 'configure') + tuple(axes)
	return _doConfigure(self, subcommand, option, kw)
    def axis_create(self, axisName, **kw):
	self.tk.call((self._w, 'axis', 'create', axisName) + self._options(kw))
    def axis_delete(self, *args):
	self.tk.call((self._w, 'axis', 'delete') + args)
    def axis_invtransform(self, axisName, value):
	return self.tk.getdouble(self.tk.call(
		self._w, 'axis', 'invtransform', axisName, value))
    def axis_limits(self, axisName):
	return self._getdoubles(self.tk.call(
		self._w, 'axis', 'limits', axisName))
    def axis_names(self, *args):
        return self.tk.splitlist(
                self.tk.call((self._w, 'axis', 'names') + args))
    def axis_transform(self, axisName, value):
	return self.tk.getint(self.tk.call(
		self._w, 'axis', 'transform', axisName, value))

    def xaxis_cget(self, key):
	return self.tk.call(self._w, 'xaxis', 'cget', '-' + key)
    def xaxis_configure(self, option=None, **kw):
	subcommand = (self._w, 'xaxis', 'configure')
	return _doConfigure(self, subcommand, option, kw)
    def xaxis_invtransform(self, value):
	return self.tk.getdouble(self.tk.call(
		self._w, 'xaxis', 'invtransform', value))
    def xaxis_limits(self):
	return self._getdoubles(self.tk.call(self._w, 'xaxis', 'limits'))
    def xaxis_transform(self, value):
	return self.tk.getint(self.tk.call(
		self._w, 'xaxis', 'transform', value))
    def xaxis_use(self, axisName = None):
	return self.tk.call(self._w, 'xaxis', 'use', axisName)

    def x2axis_cget(self, key):
	return self.tk.call(self._w, 'x2axis', 'cget', '-' + key)
    def x2axis_configure(self, option=None, **kw):
	subcommand = (self._w, 'x2axis', 'configure')
	return _doConfigure(self, subcommand, option, kw)
    def x2axis_invtransform(self, value):
	return self.tk.getdouble(self.tk.call(
		self._w, 'x2axis', 'invtransform', value))
    def x2axis_limits(self):
	return self._getdoubles(self.tk.call(self._w, 'x2axis', 'limits'))
    def x2axis_transform(self, value):
	return self.tk.getint(self.tk.call(
		self._w, 'x2axis', 'transform', value))
    def x2axis_use(self, axisName = None):
	return self.tk.call(self._w, 'x2axis', 'use', axisName)

    def yaxis_cget(self, key):
	return self.tk.call(self._w, 'yaxis', 'cget', '-' + key)
    def yaxis_configure(self, option=None, **kw):
	subcommand = (self._w, 'yaxis', 'configure')
	return _doConfigure(self, subcommand, option, kw)
    def yaxis_invtransform(self, value):
	return self.tk.getdouble(self.tk.call(
		self._w, 'yaxis', 'invtransform', value))
    def yaxis_limits(self):
	return self._getdoubles(self.tk.call(self._w, 'yaxis', 'limits'))
    def yaxis_transform(self, value):
	return self.tk.getint(self.tk.call(
		self._w, 'yaxis', 'transform', value))
    def yaxis_use(self, axisName = None):
	return self.tk.call(self._w, 'yaxis', 'use', axisName)

    def y2axis_cget(self, key):
	return self.tk.call(self._w, 'y2axis', 'cget', '-' + key)
    def y2axis_configure(self, option=None, **kw):
	subcommand = (self._w, 'y2axis', 'configure')
	return _doConfigure(self, subcommand, option, kw)
    def y2axis_invtransform(self, value):
	return self.tk.getdouble(self.tk.call(
		self._w, 'y2axis', 'invtransform', value))
    def y2axis_limits(self):
	return self._getdoubles(self.tk.call(self._w, 'y2axis', 'limits'))
    def y2axis_transform(self, value):
	return self.tk.getint(self.tk.call(
		self._w, 'y2axis', 'transform', value))
    def y2axis_use(self, axisName = None):
	return self.tk.call(self._w, 'y2axis', 'use', axisName)

    def crosshairs_cget(self, key):
	return self.tk.call(self._w, 'crosshairs', 'cget', '-' + key)
    def crosshairs_configure(self, option=None, **kw):
	subcommand = (self._w, 'crosshairs', 'configure')
	return _doConfigure(self, subcommand, option, kw)
    def crosshairs_off(self):
	self.tk.call(self._w, 'crosshairs', 'off')
    def crosshairs_on(self):
	self.tk.call(self._w, 'crosshairs', 'on')
    def crosshairs_toggle(self):
	self.tk.call(self._w, 'crosshairs', 'toggle')

    def element_activate(self, name, *args):
	self.tk.call((self._w, 'element', 'activate', name) + args)
    def element_bind(self, tagName, sequence=None, func=None, add=None):
        return self._bind((self._w, 'element', 'bind', tagName),
                sequence, func, add)
    def element_unbind(self, tagName, sequence, funcid=None):
        self.tk.call(self._w, 'element', 'bind', tagName, sequence, '')
        if funcid:
            self.deletecommand(funcid)

    def element_cget(self, name, key):
	return self.tk.call(self._w, 'element', 'cget', name, '-' + key)

    def element_closest(self, x, y, *args, **kw):
        var = 'python_private_1'
	success = self.tk.getint(self.tk.call(
                (self._w, 'element', 'closest', x, y, var) +
                        self._options(kw) + args))
	if success:
	    rtn = {}
	    rtn['dist'] = self.tk.getdouble(self.tk.globalgetvar(var, 'dist'))
	    rtn['x'] = self.tk.getdouble(self.tk.globalgetvar(var, 'x'))
	    rtn['y'] = self.tk.getdouble(self.tk.globalgetvar(var, 'y'))
	    rtn['index'] = self.tk.getint(self.tk.globalgetvar(var, 'index'))
	    rtn['name'] = self.tk.globalgetvar(var, 'name')
	    return rtn
	else:
	    return None

    def element_configure(self, names, option=None, **kw):
        # <names> may be a list of elemNames.
	if type(names) == types.StringType:
            names = [names]
	subcommand = (self._w, 'element', 'configure') + tuple(names)
	return _doConfigure(self, subcommand, option, kw)

    def element_deactivate(self, *args):
	self.tk.call((self._w, 'element', 'deactivate') + args)

    def element_delete(self, *args):
	self.tk.call((self._w, 'element', 'delete') + args)
    def element_exists(self, name):
	return self.tk.getboolean(
		self.tk.call(self._w, 'element', 'exists', name))

    def element_names(self, *args):
        return self.tk.splitlist(
                self.tk.call((self._w, 'element', 'names') + args))
    def element_show(self, nameList=None):
	if nameList is not None:
	    nameList = tuple(nameList)
	return self.tk.splitlist(
		self.tk.call(self._w, 'element', 'show', nameList))
    def element_type(self, name):
	return self.tk.call(self._w, 'element', 'type', name)

    def grid_cget(self, key):
	return self.tk.call(self._w, 'grid', 'cget', '-' + key)
    def grid_configure(self, option=None, **kw):
	subcommand = (self._w, 'grid', 'configure')
	return _doConfigure(self, subcommand, option, kw)

    def grid_off(self):
	self.tk.call(self._w, 'grid', 'off')
    def grid_on(self):
	self.tk.call(self._w, 'grid', 'on')
    def grid_toggle(self):
	self.tk.call(self._w, 'grid', 'toggle')

    def legend_activate(self, *args):
	self.tk.call((self._w, 'legend', 'activate') + args)
    def legend_bind(self, tagName, sequence=None, func=None, add=None):
        return self._bind((self._w, 'legend', 'bind', tagName),
                sequence, func, add)
    def legend_unbind(self, tagName, sequence, funcid=None):
        self.tk.call(self._w, 'legend', 'bind', tagName, sequence, '')
        if funcid:
            self.deletecommand(funcid)

    def legend_cget(self, key):
	return self.tk.call(self._w, 'legend', 'cget', '-' + key)
    def legend_configure(self, option=None, **kw):
	subcommand = (self._w, 'legend', 'configure')
	return _doConfigure(self, subcommand, option, kw)
    def legend_deactivate(self, *args):
	self.tk.call((self._w, 'legend', 'deactivate') + args)
    def legend_get(self, pos):
	return self.tk.call(self._w, 'legend', 'get', pos)

    def pen_cget(self, name, key):
	return self.tk.call(self._w, 'pen', 'cget', name, '-' + key)
    def pen_configure(self, names, option=None, **kw):
        # <names> may be a list of penNames.
	if type(names) == types.StringType:
            names = [names]
	subcommand = (self._w, 'pen', 'configure') + tuple(names)
	return _doConfigure(self, subcommand, option, kw)
    def pen_create(self, name, **kw):
	self.tk.call((self._w, 'pen', 'create', name) + self._options(kw))
    def pen_delete(self, *args):
	self.tk.call((self._w, 'pen', 'delete') + args)
    def pen_names(self, *args):
        return self.tk.splitlist(self.tk.call((self._w, 'pen', 'names') + args))

    def postscript_cget(self, key):
	return self.tk.call(self._w, 'postscript', 'cget', '-' + key)
    def postscript_configure(self, option=None, **kw):
	subcommand = (self._w, 'postscript', 'configure')
	return _doConfigure(self, subcommand, option, kw)
    def postscript_output(self, fileName=None, **kw):
	prefix = (self._w, 'postscript', 'output')
	if fileName is None:
	    return self.tk.call(prefix + self._options(kw))
	else:
	    self.tk.call(prefix + (fileName,) + self._options(kw))

    def marker_after(self, first, second=None):
	self.tk.call(self._w, 'marker', 'after', first, second)
    def marker_before(self, first, second=None):
	self.tk.call(self._w, 'marker', 'before', first, second)
    def marker_bind(self, tagName, sequence=None, func=None, add=None):
        return self._bind((self._w, 'marker', 'bind', tagName),
                sequence, func, add)
    def marker_unbind(self, tagName, sequence, funcid=None):
        self.tk.call(self._w, 'marker', 'bind', tagName, sequence, '')
        if funcid:
            self.deletecommand(funcid)

    def marker_cget(self, name, key):
	return self.tk.call(self._w, 'marker', 'cget', name, '-' + key)
    def marker_configure(self, names, option=None, **kw):
        # <names> may be a list of markerIds.
	if type(names) == types.StringType:
            names = [names]
	subcommand = (self._w, 'marker', 'configure') + tuple(names)
	return _doConfigure(self, subcommand, option, kw)
    def marker_create(self, type, **kw):
	return self.tk.call(
                (self._w, 'marker', 'create', type) + self._options(kw))

    def marker_delete(self, *args):
	self.tk.call((self._w, 'marker', 'delete') + args)
    def marker_exists(self, name):
	return self.tk.getboolean(
		self.tk.call(self._w, 'marker', 'exists', name))
    def marker_names(self, *args):
        return self.tk.splitlist(
                self.tk.call((self._w, 'marker', 'names') + args))
    def marker_type(self, name):
	type = self.tk.call(self._w, 'marker', 'type', name)
	if type == '':
	    type = None
	return type

#=============================================================================
class Stripchart(Graph):
    # Wrapper for the blt stripchart widget, version 2.4.

    def __init__(self, master=None, cnf={}, **kw):
	_loadBlt(master)
	Tkinter.Widget.__init__(self, master, _chartCommand, cnf, kw)

#=============================================================================
class Tabset(Tkinter.Widget): 

    # Wrapper for the blt TabSet widget, version 2.4.

    def __init__(self, master=None, cnf={}, **kw):
	_loadBlt(master)
	Tkinter.Widget.__init__(self, master, _tabsetCommand, cnf, kw)

    def activate(self, tabIndex):
        self.tk.call(self._w, 'activate', tabIndex)

    # This is the 'bind' sub-command:
    def tag_bind(self, tagName, sequence=None, func=None, add=None):
        return self._bind((self._w, 'bind', tagName), sequence, func, add)

    def tag_unbind(self, tagName, sequence, funcid=None):
        self.tk.call(self._w, 'bind', tagName, sequence, '')
        if funcid:
            self.deletecommand(funcid)

    def delete(self, first, last = None):
    	self.tk.call(self._w, 'delete', first, last)

    # This is the 'focus' sub-command:
    def tab_focus(self, tabIndex):
    	self.tk.call(self._w, 'focus', tabIndex)
    	
    def get(self, tabIndex):
	return self.tk.call(self._w, 'get', tabIndex)

    def index(self, tabIndex):
	index = self.tk.call(self._w, 'index', tabIndex)
        if index == '':
            return None
        else:
            return self.tk.getint(self.tk.call(self._w, 'index', tabIndex))

    def insert(self, position, name1, *names, **kw):
	self.tk.call(
            (self._w, 'insert', position, name1) + names + self._options(kw))

    def invoke(self, tabIndex):
    	return self.tk.call(self._w, 'invoke', tabIndex)

    def move(self, tabIndex1, beforeOrAfter, tabIndex2):
    	self.tk.call(self._w, 'move', tabIndex1, beforeOrAfter, tabIndex2)
    	
    def nearest(self, x, y):
    	return self.tk.call(self._w, 'nearest', x, y)

    def scan_mark(self, x, y):
        self.tk.call(self._w, 'scan', 'mark', x, y)

    def scan_dragto(self, x, y):
        self.tk.call(self._w, 'scan', 'dragto', x, y)

    def see(self, index):
        self.tk.call(self._w, 'see', index)
    	
    def see(self, tabIndex):
    	self.tk.call(self._w,'see',tabIndex)
    	
    def size(self):
	return self.tk.getint(self.tk.call(self._w, 'size'))

    def tab_cget(self, tabIndex, option):
        if option[:1] != '-':
            option = '-' + option
        if option[-1:] == '_':
            option = option[:-1]
        return self.tk.call(self._w, 'tab', 'cget', tabIndex, option)

    def tab_configure(self, tabIndexes, option=None, **kw):
        # <tabIndexes> may be a list of tabs.
	if type(tabIndexes) in (types.StringType, types.IntType):
            tabIndexes = [tabIndexes]
	subcommand = (self._w, 'tab', 'configure') + tuple(tabIndexes)
	return _doConfigure(self, subcommand, option, kw)

    def tab_names(self, *args):
        return self.tk.splitlist(self.tk.call((self._w, 'tab', 'names') + args))

    def tab_tearoff(self, tabIndex, newName = None):
        if newName is None:
            name = self.tk.call(self._w, 'tab', 'tearoff', tabIndex)
            return self.nametowidget(name)
        else:
            self.tk.call(self._w, 'tab', 'tearoff', tabIndex, newName)

    def view(self):
        s = self.tk.call(self._w, 'view')
        return tuple(map(self.tk.getint, self.tk.splitlist(s)))
    def view_moveto(self, fraction):
        self.tk.call(self._w, 'view', 'moveto', fraction)
    def view_scroll(self, number, what):
        self.tk.call(self._w, 'view', 'scroll', number, what)



import types
import Tkinter
import Pmw

class RadioSelect(Pmw.MegaWidget):
    # A collection of several buttons.  In single mode, only one
    # button may be selected.  In multiple mode, any number of buttons
    # may be selected.

    def __init__(self, parent = None, **kw):

	# Define the megawidget options.
	INITOPT = Pmw.INITOPT
	optiondefs = (
	    ('buttontype',    'button',      INITOPT),
	    ('command',       None,          None),
	    ('labelmargin',   0,             INITOPT),
	    ('labelpos',      None,          INITOPT),
	    ('orient',       'horizontal',   INITOPT),
	    ('padx',          5,             INITOPT),
	    ('pady',          5,             INITOPT),
	    ('selectmode',    'single',      INITOPT),
	)
	self.defineoptions(kw, optiondefs, dynamicGroups = ('Button',))

	# Initialise the base class (after defining the options).
	Pmw.MegaWidget.__init__(self, parent)

	# Create the components.
	interior = self.interior()
	if self['labelpos'] is None:
	    self._radioSelectFrame = self._hull
	else:
	    self._radioSelectFrame = self.createcomponent('frame',
		    (), None,
		    Tkinter.Frame, (interior,))
	    self._radioSelectFrame.grid(column=2, row=2, sticky='nsew')
	    interior.grid_columnconfigure(2, weight=1)
	    interior.grid_rowconfigure(2, weight=1)

	    self.createlabel(interior)

	# Initialise instance variables.
	self._buttonList = []
	if self['selectmode'] == 'single':
	    self._singleSelect = 1
	elif self['selectmode'] == 'multiple':
	    self._singleSelect = 0
	else: 
	    raise ValueError, 'bad selectmode option "' + \
		    self['selectmode'] + '": should be single or multiple'

	if self['buttontype'] == 'button':
	    self.buttonClass = Tkinter.Button
	elif self['buttontype'] == 'radiobutton':
	    self._singleSelect = 1
	    self.var = Tkinter.StringVar()
	    self.buttonClass = Tkinter.Radiobutton
	elif self['buttontype'] == 'checkbutton':
	    self._singleSelect = 0
	    self.buttonClass = Tkinter.Checkbutton
	else:
	    raise ValueError, 'bad buttontype option "' + \
		    self['buttontype'] + \
		    '": should be button, radiobutton or checkbutton'

	if self._singleSelect:
	    self.selection = None
	else:
	    self.selection = []

	if self['orient'] not in ('horizontal', 'vertical'):
	    raise ValueError, 'bad orient option ' + repr(self['orient']) + \
		': must be either \'horizontal\' or \'vertical\''

	# Check keywords and initialise options.
	self.initialiseoptions()

    def getcurselection(self):
	if self._singleSelect:
            return self.selection
        else:
            return tuple(self.selection)

    def getvalue(self):
        return self.getcurselection()

    def setvalue(self, textOrList):
	if self._singleSelect:
            self.__setSingleValue(textOrList)
        else:
	    # Multiple selections
            oldselection = self.selection
            self.selection = textOrList
            for button in self._buttonList:
                if button in oldselection:
                    if button not in self.selection:
                        # button is currently selected but should not be
                        widget = self.component(button)
                        if self['buttontype'] == 'checkbutton':
                            widget.deselect()
                        else:  # Button
                            widget.configure(relief='raised')
                else:
                    if button in self.selection:
                        # button is not currently selected but should be
                        widget = self.component(button)
                        if self['buttontype'] == 'checkbutton':
                            widget.select()
                        else:  # Button
                            widget.configure(relief='sunken')

    def numbuttons(self):
        return len(self._buttonList)

    def index(self, index):
	# Return the integer index of the button with the given index.

	listLength = len(self._buttonList)
	if type(index) == types.IntType:
	    if index < listLength:
		return index
	    else:
		raise ValueError, 'index "%s" is out of range' % index
	elif index is Pmw.END:
	    if listLength > 0:
		return listLength - 1
	    else:
		raise ValueError, 'RadioSelect has no buttons'
	else:
	    for count in range(listLength):
		name = self._buttonList[count]
		if index == name:
		    return count
	    validValues = 'a name, a number or Pmw.END'
	    raise ValueError, \
		    'bad index "%s": must be %s' % (index, validValues)

    def button(self, buttonIndex):
	name = self._buttonList[self.index(buttonIndex)]
        return self.component(name)

    def add(self, componentName, **kw):
	if componentName in self._buttonList:
	    raise ValueError, 'button "%s" already exists' % componentName

	kw['command'] = \
                lambda self=self, name=componentName: self.invoke(name)
	if not kw.has_key('text'):
	    kw['text'] = componentName

	if self['buttontype'] == 'radiobutton':
	    if not kw.has_key('anchor'):
		kw['anchor'] = 'w'
	    if not kw.has_key('variable'):
		kw['variable'] = self.var
	    if not kw.has_key('value'):
		kw['value'] = kw['text']
	elif self['buttontype'] == 'checkbutton':
	    if not kw.has_key('anchor'):
		kw['anchor'] = 'w'

	button = apply(self.createcomponent, (componentName,
		(), 'Button',
		self.buttonClass, (self._radioSelectFrame,)), kw)

	if self['orient'] == 'horizontal':
	    self._radioSelectFrame.grid_rowconfigure(0, weight=1)
	    col = len(self._buttonList)
	    button.grid(column=col, row=0, padx = self['padx'],
		    pady = self['pady'], sticky='nsew')
	    self._radioSelectFrame.grid_columnconfigure(col, weight=1)
	else:
	    self._radioSelectFrame.grid_columnconfigure(0, weight=1)
	    row = len(self._buttonList)
	    button.grid(column=0, row=row, padx = self['padx'],
		    pady = self['pady'], sticky='ew')
	    self._radioSelectFrame.grid_rowconfigure(row, weight=1)

	self._buttonList.append(componentName)
	return button

    def deleteall(self):
	for name in self._buttonList:
	    self.destroycomponent(name)
	self._buttonList = []
	if self._singleSelect:
	    self.selection = None
	else: 
	    self.selection = []

    def __setSingleValue(self, value):
            self.selection = value
            if self['buttontype'] == 'radiobutton':
                widget = self.component(value)
                widget.select()
            else:  # Button
                for button in self._buttonList:
                    widget = self.component(button)
                    if button == value:
                        widget.configure(relief='sunken')
                    else:
                        widget.configure(relief='raised')

    def invoke(self, index):
	index = self.index(index)
	name = self._buttonList[index]

	if self._singleSelect:
            self.__setSingleValue(name)
	    command = self['command']
	    if callable(command):
		return command(name)
        else:
	    # Multiple selections
	    widget = self.component(name)
	    if name in self.selection:
		if self['buttontype'] == 'checkbutton':
		    widget.deselect()
		else:
		    widget.configure(relief='raised')
		self.selection.remove(name)
		state = 0
	    else:
		if self['buttontype'] == 'checkbutton':
		    widget.select()
		else:
		    widget.configure(relief='sunken')
		self.selection.append(name)
		state = 1

	    command = self['command']
	    if callable(command):
	      return command(name, state)



"""Handle command line arguments.

This module contains functions to parse and access the arguments given
to the program on the command line.
"""

import types
import string
import sys

# Symbolic constants for the indexes into an argument specifier tuple.
NAME        = 0
MANDATORY   = 1
TYPE        = 2
HELP        = 3
DEFAULT     = 4
SPEC_LENGTH = 5

Bool = []

helpSpec = (
  ('help',            0, Bool,   'print help and exit'),
)

def parseArgs(title, argv, argSpecs, filesOK):
  """Parse and check command line arguments.

  Scan the command line arguments in *argv* according to the argument
  specifier *argSpecs*. Return **None** if there are no errors in
  the arguments, otherwise return an error string describing the error.

  This function must be called to initialise this module.

  title    -- The name of the program. This is used when returning
              error messages or help text.

  argv     -- A sequence containing the arguments given to the program.
              Normally **sys.argv**.

  argSpecs -- A sequence of argument specifiers.  Each specifier describes
              a valid command line argument and consists of 4 or 5 items:

	      - The argument name (without a leading minus sign **-**).

	      - A boolean value, true if the argument is mandatory.

	      - This should be **Args.Bool** if the argument has no option.
		Otherwise it should be a string describing the option
		required for this argument. This is used when printing help.

	      - A short string describing the argument.

	      - The default value of the argument.  This should only be used
	        for non-mandatory arguments expecting an option.
	      
	      For example:
		(
		  ('foreground', 0, 'colour',    'colour of text', 'black'),
		  ('geometry',   0, 'spec',      'geometry of initial window'),
		  ('server',     1, 'ompserver', 'ompserver to connect to'),
		  ('silent',     0, Args.Bool,   'do not sound bell'),
		)
  """

  global programName
  global _fileList

  errMsg = title + ' command line error: '
  programName = argv[0];

  argSpecs = helpSpec + argSpecs
  argSpecDic = {}
  for spec in argSpecs:
    arg = spec[NAME]
    argSpecDic[arg] = spec
    if len(spec) >= SPEC_LENGTH:
      set(arg, spec[DEFAULT])
    elif spec[TYPE] is Bool:
      set(arg, 0)
    else:
      set(arg, None)

  knownKeys = argSpecDic.keys()

  i = 1
  _fileList = []
  argc = len(argv)
  while i < argc:
    arg = argv[i]
    key = arg[1:]
    if key in knownKeys:
      spec = argSpecDic[key]
      if spec[TYPE] is Bool:
	set(key, 1)
      else:
	i = i + 1
	if i >= argc:
	  return errMsg + 'missing argument to \'' + arg + '\' option.'
	value = argv[i]
	if len(spec) >= SPEC_LENGTH:
	  try:
	      if type(spec[DEFAULT]) == types.IntType:
		  typeStr = 'integer'
		  value = string.atoi(value)
	      elif type(spec[DEFAULT]) == types.FloatType:
		  typeStr = 'float'
		  value = string.atof(value)
	  except:
	      sys.exc_traceback = None   # Clean up object references
	      return errMsg + 'cannot convert string \'' + value + \
	        '\' to ' + typeStr + ' for option \'-' + key + '\'.'
	set(key, value)
    else:
      _fileList.append(arg)
    i = i + 1

  if get('help'):
    return _helpString(title, argSpecs)

  if not filesOK and len(_fileList) > 0:
    if len(_fileList) == 1:
      return errMsg + 'unknown option \'' + str(_fileList[0]) + '\'.'
    else:
      return errMsg + 'unknown options ' + str(_fileList) + '.'


  _missing = []
  for spec in argSpecs:
    if spec[MANDATORY] and get(spec[NAME]) is None:
      _missing.append(spec[NAME])
  if len(_missing) == 1:
    return errMsg + 'required argument \'-' + \
      str(_missing[0]) + '\' is missing.'
  elif len(_missing) > 1:
    return errMsg + 'required arguments ' + \
      str(map(lambda s: '-' + s, _missing)) + ' are missing.'

  return None

def fileList():
  return _fileList

def _helpString(title, argSpecs):
  max = 0
  for spec in argSpecs:
    if spec[TYPE] is Bool:
      width = len(spec[NAME]) + 1
    else:
      width = len(spec[NAME]) + 4 + len(spec[TYPE])
    if width > max:
      max = width

  rtn = title + ' command line arguments:'
  format = '\n  %-' + str(max) + 's %s'
  for mandatory in (1, 0):
    needHeader = 1
    for spec in argSpecs:
      if mandatory and spec[MANDATORY] or not mandatory and not spec[MANDATORY]:
	if needHeader:
	  if mandatory:
	    rtn = rtn + '\n Mandatory arguments:'
	  else:
	    rtn = rtn + '\n Optional arguments (defaults in parentheses):'
	  needHeader = 0
	if spec[TYPE] is Bool:
	  arg = '-%s' % spec[NAME]
	else:
	  arg = '-%s <%s>' % (spec[NAME], spec[TYPE])
	if len(spec) >= SPEC_LENGTH:
	  if type(spec[DEFAULT]) == types.StringType:
	      definition = spec[HELP] + ' (' + spec[DEFAULT] + ')'
	  else:
	      definition = spec[HELP] + ' (' + str(spec[DEFAULT]) + ')'
	else:
	  definition = spec[HELP]
	rtn = rtn + format % (arg, definition)

  return rtn

def exists(key):
    return configDict.has_key(key)

def get(key):
    return configDict[key]

def set(key, value):
    global configDict

    configDict[key] = value

configDict = {}
